"""Clothing sizes data provider."""

from mimesis.providers.base import BaseDataProvider

__all__ = ['ClothingSize']


class ClothingSize(BaseDataProvider):
    """Class for generate clothing sizes data."""

    def international_size(self) -> str:
        """Get a random size in international format.

        :return: Clothing size.

        :Example:
            XXL.
        """
        return self.random.choice(['L', 'M', 'S', 'XL',
                                   'XS', 'XXL', 'XXS', 'XXXL'])

    def european_size(self) -> int:
        """Generate a random clothing size in European format.

        :return: Clothing size.

        :Example:
            42
        """
        return self.random.randint(38, 62)

    def custom_size(self, minimum: int = 40, maximum: int = 62) -> int:
        """Generate clothing size using custom format.

        :param minimum: Minimum value.
        :param maximum: Maximum value.
        :return: Clothing size.

        :Example:
            44
        """
        return self.random.randint(minimum, maximum)



"""Base specific data provider."""

from mimesis.providers import BaseProvider

__all__ = ['BaseSpecProvider']


class BaseSpecProvider(BaseProvider):
    """Base provider for specific data providers."""

    def __init__(self, *args, **kwargs):
        """Initialize attributes of superclass."""
        super().__init__(*args, **kwargs)



"""Cryptographic data provider."""

import hashlib
import string
import uuid
from typing import Optional

from mimesis.enums import Algorithm
from mimesis.providers.base import BaseDataProvider
from mimesis.providers.text import Text
from mimesis.typing import Bytes

__all__ = ['Cryptographic']


class Cryptographic(BaseDataProvider):
    """Class that provides cryptographic data."""

    def __init__(self, *args, **kwargs) -> None:
        """Initialize attributes.

        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)
        self.__words = Text('en')._data['words']
        self.__chars = string.ascii_letters + string.digits + string.punctuation

    def uuid(self, version: Optional[int] = None) -> str:
        """Generate random UUID.

        :param version: UUID version.
        :return: UUID
        """
        bits = self.random.getrandbits(128)
        return str(uuid.UUID(int=bits, version=version))

    def hash(self, algorithm: Optional[Algorithm] = None) -> str:
        """Generate random hash.

        :param algorithm: Enum object ``Algorithm``.
        :return: Hash.
        :raises NonEnumerableError: if algorithm is not supported.
        """
        key = self._validate_enum(algorithm, Algorithm)

        if hasattr(hashlib, key):
            fn = getattr(hashlib, key)
            return fn(self.uuid().encode()).hexdigest()

    def bytes(self, entropy: int = 32) -> Bytes:
        """Generate byte string containing *entropy* bytes.

        The string has *entropy* random bytes, each byte
        converted to two hex digits.

        :param entropy: Number of bytes.
        :return: Bytes.
        :rtype: bytes
        """
        return bytes(self.random.getrandbits(8)
                     for _ in range(entropy))

    def token(self, entropy: int = 32) -> str:
        """Generate hexadecimal string.

        :param entropy: Number of bytes.
        :return: Token.
        """
        return self.bytes(entropy).hex()

    def salt(self, size: int = 16) -> str:
        """Generate salt chars (not cryptographically safe).

        :param size: Salt size.
        :return: Salt.
        """
        char_sequence = [
            self.random.choice(self.__chars)
            for _ in range(size)
        ]
        return ''.join(char_sequence)

    def mnemonic_phrase(self, length: int = 12) -> str:
        """Generate pseudo mnemonic phrase.

        :param length: Number of words.
        :return: Mnemonic code.
        """
        words = self.__words['normal']
        return ' '.join(self.random.choice(words) for _ in range(length))



"""Base data provider."""

from typing import Any, Optional

from mimesis.exceptions import NonEnumerableError
from mimesis.helpers import Random, get_random_item
from mimesis.typing import Seed
from mimesis.utils import setup_locale

__all__ = ['BaseDataProvider', 'BaseProvider']


class BaseProvider(object):
    """This is a base class for all providers."""

    def __init__(self, seed: Optional[Seed] = None) -> None:
        """Initialize attributes.

        :param seed: Seed for random.
        """
        self.seed = seed
        self.random = Random()

        if seed is not None:
            self.random.seed(self.seed)

    def _validate_enum(self, item: Any, enum: Any) -> Any:
        """Validate enum parameter of method in subclasses of BaseProvider.

        :param item: Item of enum object.
        :param enum: Enum object.
        :return: Value of item.
        :raises NonEnumerableError: if ``item`` not in ``enum``.
        """
        if item is None:
            result = get_random_item(enum, self.random)
        elif item and isinstance(item, enum):
            result = item
        else:
            raise NonEnumerableError(enum)

        return result.value


class StrMixin(object):
    """A mixin for showing information about the current locale."""

    def __str__(self) -> str:
        """Human-readable representation of locale."""
        locale = getattr(self, 'locale', 'en')
        return '{} <{}>'.format(
            self.__class__.__name__, locale)


class BaseDataProvider(BaseProvider, StrMixin):
    """This is a base class for all data providers."""

    def __init__(self, locale: Optional[str] = None,
                 seed: Optional[Seed] = None) -> None:
        """Initialize attributes for data providers.

        :param locale: Current locale.
        :param seed: Seed to all the random functions.
        """
        super().__init__(seed=seed)
        self.locale = setup_locale(locale)

    def get_current_locale(self) -> str:
        """Get current locale.

        If locale is not defined then this method will always return ``en``,
        because ``en`` is default locale for all providers, excluding builtins.

        :return: Current locale.
        """
        return self.locale



"""Specific data provider for USA (en)."""

from typing import Union

from mimesis.builtins.base import BaseSpecProvider

__all__ = ['USASpecProvider']


class USASpecProvider(BaseSpecProvider):
    """Class that provides special data for USA (en)."""

    class Meta:
        """The name of the provider."""

        name = 'usa_provider'

    def tracking_number(self, service: str = 'usps') -> str:
        """Generate random tracking number.

        Supported services: USPS, FedEx and UPS.

        :param str service: Post service.
        :return: Tracking number.
        """
        service = service.lower()

        if service not in ('usps', 'fedex', 'ups'):
            raise ValueError('Unsupported post service')

        services = {
            'usps': (
                '#### #### #### #### ####',
                '@@ ### ### ### US',
            ),
            'fedex': (
                '#### #### ####',
                '#### #### #### ###',
            ),
            'ups': (
                '1Z@####@##########',
            ),
        }
        mask = self.random.choice(services[service])  # type: ignore
        return self.random.custom_code(mask=mask)

    def ssn(self) -> str:
        """Generate a random, but valid SSN.

        :returns: SSN.

        :Example:
            569-66-5801
        """
        area = self.random.randint(1, 899)
        if area == 666:
            area = 665

        return '{:03}-{:02}-{:04}'.format(
            area,
            self.random.randint(1, 99),
            self.random.randint(1, 9999),
        )

    def personality(self, category: str = 'mbti') -> Union[str, int]:
        """Generate a type of personality.

        :param category: Category.
        :return: Personality type.
        :rtype: str or int

        :Example:
            ISFJ.
        """
        mbtis = ('ISFJ', 'ISTJ', 'INFJ', 'INTJ',
                 'ISTP', 'ISFP', 'INFP', 'INTP',
                 'ESTP', 'ESFP', 'ENFP', 'ENTP',
                 'ESTJ', 'ESFJ', 'ENFJ', 'ENTJ')

        if category.lower() == 'rheti':
            return self.random.randint(1, 10)

        return self.random.choice(mbtis)



"""Business data provider."""

from mimesis.data import (CRYPTOCURRENCY_ISO_CODES, CRYPTOCURRENCY_SYMBOLS,
                          CURRENCY_ISO_CODES, CURRENCY_SYMBOLS)
from mimesis.providers.base import BaseDataProvider
from mimesis.utils import pull

__all__ = ['Business']


class Business(BaseDataProvider):
    """Class for generating data for business."""

    def __init__(self, *args, **kwargs):
        """Initialize attributes.

        :param locale: Current locale.
        """
        super().__init__(*args, **kwargs)
        self._data = pull('business.json', self.locale)

    def company(self) -> str:
        """Get a random company name.

        :return: Company name.

        :Example:
            Gamma Systems.
        """
        return self.random.choice(
            self._data['company']['name'])

    def company_type(self, abbr: bool = False) -> str:
        """Get a random type of business entity.

        :param abbr: Abbreviated company type.
        :return: Types of business entity.

        :Example:
            Incorporated.
        """
        return self.random.choice(
            self._data['company']['type'].get(
                'abbr' if abbr else 'title'),
        )

    def copyright(self) -> str:
        """Generate a random copyright.

        :return: Copyright of company.

        :Example:
            © Komercia, Inc.
        """
        return '© {}, {}'.format(
            self.company(),
            self.company_type(abbr=True),
        )

    def currency_iso_code(self) -> str:
        """Get code of the currency.

        :return: Currency code.

        :Example:
            RUR.
        """
        return self.random.choice(CURRENCY_ISO_CODES)

    def cryptocurrency_iso_code(self) -> str:
        """Get symbol of random cryptocurrency.

        :return: Symbol of cryptocurrency.
        """
        return self.random.choice(CRYPTOCURRENCY_ISO_CODES)

    def currency_symbol(self):
        """Get a currency symbol for current locale.

        :return: Currency symbol.
        """
        return CURRENCY_SYMBOLS[self.locale]

    def cryptocurrency_symbol(self) -> str:
        """Get a cryptocurrency symbol.

        :return: Symbol of cryptocurrency.

        :Example:
            Ƀ
        """
        return self.random.choice(CRYPTOCURRENCY_SYMBOLS)

    def price(self, minimum: float = 10.00,
              maximum: float = 1000.00) -> str:
        """Generate a random price.

        :param minimum: Max value of price.
        :param maximum: Min value of price.
        :return: Price.

        :Example:
            599.99 $.
        """
        price = self.random.uniform(minimum, maximum, precision=2)
        return '{0} {1}'.format(price, self.currency_symbol())

    def price_in_btc(self, minimum: float = 0, maximum: float = 2) -> str:
        """Generate random price in BTC.

        :param minimum: Minimum value of price
        :param maximum: Maximum value of price.
        :return: Price in BTC.

        :Example:
            0.5885238 BTC
        """
        return '{} BTC'.format(
            self.random.uniform(
                minimum,
                maximum,
                precision=7,
            ),
        )



"""Provides platforms."""

# https://docs.python.org/3.5/library/sys.html?highlight=sys#sys.platform
PLATFORMS = {
    'LINUX': {
        'name': 'linux',
        'path_separator': '/',
        'root': '/',
        'home': '/home/',
    },
    'MACOS': {
        'name': 'darwin',
        'path_separator': '/',
        'root': '/',
        'home': '/home/',
    },
    'WINDOWS': {
        'name': 'win32',
        'path_separator': '\\',
        'root': 'C:\\',
        'home': 'C:\\Users\\',
    },
    'WINDOWS64': {
        'name': 'win64',
        'path_separator': '\\',
        'root': 'C:\\',
        'home': 'C:\\Users\\',
    },
}



"""Provides all the data related to transports."""

TRUCKS = [
    'ACMAT',
    'AEC',
    'AIL',
    'AWD',
    'Albion',
    'Argyle',
    'Askam',
    'Astra',
    'Atkinson',
    'Austin',
    'Automiesse',
    'BMC',
    'Barkas',
    'Baron',
    'Barreiros',
    'Bean',
    'Beardmore',
    'Bedford',
    'BelAZ',
    'Berliet',
    'Berna',
    'Bernard',
    'Borgward',
    'Bristol',
    'Brossel',
    'Bucegi',
    'Büssing',
    'Caledon',
    'Carmichael',
    'Chavdar',
    'Chery',
    'Chingkangshan',
    'Citroen',
    'Commer',
    'Crossley',
    'Csepel',
    'Daewoo',
    'Daihatsu',
    'Delahaye',
    'Dennison',
    'Dodge',
    'Douglas',
    'ELBO',
    'ERF',
    'Ebro',
    'FAP',
    'FAW',
    'FBW',
    'Fargo',
    'Fiat',
    'Foden',
    'Ford',
    'Foton',
    'Fowler',
    'GA',
    'GAZ',
    'GV',
    'Garner',
    'Garrett',
    'Gaz',
    'Genoto',
    'Gilford',
    'Ginaf',
    'Grube',
    'Guy',
    'HHT',
    'HSG',
    'Halley',
    'Hallford',
    'Hanomag',
    'Hardy',
    'Haulamatic',
    'Henschel',
    'Hino',
    'Horch',
    'Hotchkiss',
    'Hyundai',
    'IFA',
    'Isuzu',
    'Iveco',
    'Jelcz',
    'Jensen',
    'Jiaotong',
    'Jiefang',
    'KAZ',
    'KMC',
    'Kaelble',
    'Kalmar',
    'Kamaz',
    'Karrier',
    'Komatsu',
    'KrAZ',
    'LAZ',
    'LIAZ',
    'Labourier',
    'Lacre',
    'Lancia',
    'Latil',
    'Leyland',
    'Loheac',
    'Lomount',
    'MAN',
    'MAZ',
    'MTN',
    'MTZ',
    'MZKT',
    'Madara',
    'Magirus',
    'Manchester',
    'Mann',
    'Maudslay',
    'Mazda',
    'McCrud',
    'Minerva',
    'Minsei',
    'Morris',
    'Mowag',
    'Multicar',
    'Multiwheeler',
    'Namco',
    'Neobus',
    'Nibus',
    'Nicolas',
    'Nissian',
    'Norde',
    'Opel',
    'Otokar',
    'Otosan',
    'Pagefield',
    'Panhard',
    'Pegaso',
    'Peugeot',
    'Praga',
    'Preslav',
    'Proctor',
    'Quest',
    'Raba',
    'Robur',
    'Roman',
    'Rotinoff',
    'Rutland',
    'SD',
    'SR',
    'Saurer',
    'Saviem',
    'Scammell',
    'Scania',
    'Seddon',
    'Sentinel',
    'Shacman',
    'Shefflex',
    'Sides',
    'Simca',
    'Sinotruk',
    'Sitom',
    'Somua',
    'Star',
    'Steyr',
    'Straussler',
    'TAM',
    'TVM',
    'TVS',
    'TVW',
    'Tata',
    'Tatra',
    'Temax',
    'Terberg',
    'Thornycroft',
    'Titan',
    'Tonar',
    'URO',
    'Unic',
    'Unimog',
    'Union',
    'Unipower',
    'Ural',
    'Vanaja',
    'Verheul',
    'Volkswagen',
    'Vomag',
    'Vulcan',
    'Willeme',
    'XCMG',
    'Yarovit',
    'Yorkshire',
    'Yuejin',
    'ZiL',
    'Zwicky',
    'ÖAF',
    'Škoda',
]

CARS = [
    'Ford Model T',
    'Volkswagen Beetle',
    'Toyota Corolla',
    'Ford Falcon',
    'Volkswagen Gol',
    'Volkswagen Santana',
    'Škoda Octavia',
    'Renault Clio',
    'Volkswagen Golf',
    'Hindustan Ambassador',
    'Fiat Uno',
    'Toyota Corolla',
    'Perodua Myvi',
    'SEAT Ibiza',
    'Volvo 200 Series',
    'Renault Symbol',
    'ZAZ Zaporozhets',
    'Morris Mini',
    'Ford F-Series',
    'Alfa Romeo Alfasud',
    'Alpine A310',
    'AMC Hornet',
    'Aston Martin DB7',
    'Autobianchi A112',
    'BMW 3 Series',
    'Buick LeSabre',
    'Bugatti Type 40',
    'Cadillac De Ville',
    'Checker Marathon',
    'Chevrolet Impala',
    'Chrysler Newport',
    'Citroën 2CV',
    'Continental Mark II',
    'Crosley',
    'DeSoto Custom',
    'De Tomaso Pantera',
    'Dodge Coronet',
    'Eagle Talon',
    'Edsel Ranger',
    'Excalibur Series II',
    'Facellia',
    'Ferrari 360',
    'Fiat Uno',
    'Ford F-Series',
    'Frazer Standard',
    'Henry J Deluxe',
    'Hindustan Ambassador',
    'Holden Commodore',
    'Honda Civic',
    'Hudson Super',
    'Imperial Crown',
    'Jaguar XJ',
    'Jeep Cherokee (XJ)',
    'Kaiser Deluxe',
    'Koenigsegg CCX',
    'Lada Riva',
    'Series/Defender',
    'Lamborghini Gallardo',
    'Lancia Ypsilon',
    'Lincoln Town Car',
    'Lotus Elise',
    'McLaren 12C',
    'Maserati Biturbo',
    'Mazda Familia',
    'Mercedes-Benz C-Class',
    'Mercury Grand Marquis',
    'Messerschmitt KR200',
    'Metropolitan',
    'Mitsubishi Lancer',
    'Nash Statesman',
    'Oldsmobile Cutlass',
    'Opel Corsa',
    'Packard Eight',
    'Peel Trident',
    'Perodua Myvi',
    'Peugeot 206',
    'Plymouth Fury',
    'Pontiac Grand Am',
    'Porsche 911',
    'Rambler Classic',
    'Renault Clio',
    'Rolls-Royce Silver Shadow',
    'Saab 900',
    'Saturn S-Series',
    'SEAT Ibiza',
    'Simca 1100',
    'Smart Fortwo',
    'Studebaker Champion',
    'Subaru Legacy',
    'Škoda Octavia',
    'Toyota Corolla',
    'Tesla Model S',
    'Trabant',
    'Volkswagen Golf',
    'Volvo 200 Series',
    'Willys 77',
    'Zaporozhets',
    'Nissan Leaf',
    'Chevrolet Impala',
    'Toyota Prius',
    'Ford F-Series',
    'Chevrolet Volt',
    'Cadillac De Ville',
    'Porsche 911 GT3 Cup',
    'Mazda RX-7',
    'Nissan Z-cars',
    'Mazda MX-5',
    'Volkswagen Type 2',
]

AIRPLANES = (
    'Airbus',
    'Boeing',
    'Tu',
)

VR_CODES = [
    'A',
    'ABH',
    'AFG',
    'AG',
    'AL',
    'AM',
    'AND',
    'ANG',
    'ARK',
    'AS',
    'AUA',
    'AUS',
    'AX',
    'AXA',
    'AZ',
    'B',
    'BD',
    'BDS',
    'BF',
    'BG',
    'BH',
    'BHT',
    'BIH',
    'BOL',
    'BR',
    'BRN',
    'BRU',
    'BS',
    'BUR',
    'BVI',
    'BW',
    'BY',
    'BZ',
    'C',
    'CAM',
    'CDN',
    'CGO',
    'CH',
    'CHN',
    'CI',
    'CL',
    'CO',
    'COM',
    'CR',
    'CV',
    'CY',
    'CYM',
    'CZ',
    'D',
    'DJI',
    'DK',
    'DOM',
    'DY',
    'DZ',
    'E',
    'EAK',
    'EAT',
    'EAU',
    'EAZ',
    'EC',
    'EIR',
    'ENG',
    'ER',
    'ES',
    'EST',
    'ET',
    'ETH',
    'F',
    'FIN',
    'FJI',
    'FL',
    'FO',
    'FSM',
    'G',
    'GB',
    'GBA',
    'GBG',
    'GBJ',
    'GBM',
    'GBZ',
    'GCA',
    'GE',
    'GH',
    'GQ',
    'GR',
    'GUY',
    'GW',
    'H',
    'HK',
    'HKJ',
    'HN',
    'HR',
    'I',
    'IL',
    'IND',
    'IR',
    'IRL',
    'IRQ',
    'IS',
    'J',
    'JA',
    'K',
    'KAN',
    'KIR',
    'KN',
    'KP',
    'KS',
    'KSA',
    'KWT',
    'KZ',
    'L',
    'LAO',
    'LAR',
    'LB',
    'LS',
    'LT',
    'LV',
    'M',
    'MA',
    'MAL',
    'MC',
    'MD',
    'MEX',
    'MGL',
    'MH',
    'MK',
    'MNE',
    'MO',
    'MOC',
    'MS',
    'MV',
    'MW',
    'N',
    'NA',
    'NAM',
    'NAU',
    'NC',
    'NEP',
    'NI',
    'NIC',
    'NL',
    'NZ',
    'OM',
    'P',
    'PA',
    'PAL',
    'PE',
    'PK',
    'PL',
    'PMR',
    'PNG',
    'PR',
    'PS',
    'PY',
    'Q',
    'RA',
    'RB',
    'RC',
    'RCA',
    'RCB',
    'RCH',
    'RG',
    'RH',
    'RI',
    'RIM',
    'RKS',
    'RL',
    'RM',
    'RMM',
    'RN',
    'RO',
    'ROK',
    'RP',
    'RSM',
    'RSO',
    'RU',
    'RUS',
    'RWA',
    'S',
    'SCO',
    'SCV',
    'SD',
    'SGP',
    'SK',
    'SLE',
    'SLO',
    'SME',
    'SMOM',
    'SN',
    'SO',
    'SOL',
    'SRB',
    'STP',
    'SUD',
    'SY',
    'SYR',
    'T',
    'TCH',
    'TG',
    'TJ',
    'TL',
    'TM',
    'TN',
    'TO',
    'TR',
    'TT',
    'TUV',
    'UA',
    'UAE',
    'USA',
    'UY',
    'UZ',
    'V',
    'VN',
    'VU',
    'WAG',
    'WAL',
    'WAN',
    'WD',
    'WG',
    'WL',
    'WS',
    'WSA',
    'WV',
    'YAR',
    'YV',
    'Z',
    'ZA',
    'ZW',
]

VRC_BY_LOCALES = {
    'cs': 'CZ',
    'da': 'DK',
    'de': 'D',
    'de-at': 'A',
    'de-ch': 'CH',
    'el': 'GR',
    'en': 'USA',
    'en-au': 'AUS',
    'en-ca': 'CDN',
    'en-gb': 'GB',
    'es': 'E',
    'es-mx': 'MEX',
    'et': 'EW',
    'fa': 'IR',
    'fi': 'FIN',
    'fr': 'F',
    'hu': 'H',
    'is': 'IS',
    'it': 'I',
    'ja': 'J',
    'kk': 'KZ',
    'ko': 'ROK',
    'nl': 'NL',
    'nl-be': 'B',
    'no': 'N',
    'pl': 'PL',
    'pt': 'P',
    'pt-br': 'BR',
    'ru': 'RUS',
    'sv': 'S',
    'tr': 'TR',
    'uk': 'UA',
    'zh': 'CHN',
}



"""Specific data provider for Netherlands (nl)."""

from mimesis.builtins.base import BaseSpecProvider

__all__ = ['NetherlandsSpecProvider']


class NetherlandsSpecProvider(BaseSpecProvider):
    """Class that provides special data for Netherlands (nl)."""

    class Meta:
        """The name of the provider."""

        name = 'netherlands_provider'

    def bsn(self) -> str:
        """Generate a random, but valid ``Burgerservicenummer``.

        :returns: Random BSN.

        :Example:
            255159705
        """
        def _is_valid_bsn(number: str) -> bool:
            total = 0
            multiplier = 9

            for char in number:
                multiplier = -multiplier if multiplier == 1 else multiplier
                total += int(char) * multiplier
                multiplier -= 1

            result = total % 11 == 0
            return result

        a, b = (100000000, 999999999)
        sample = str(self.random.randint(a, b))

        while not _is_valid_bsn(sample):
            sample = str(self.random.randint(a, b))

        return sample

    def burgerservicenummer(self) -> str:
        """Generate a random, but valid ``Burgerservicenummer``.

        An alias for self.bsn()
        """
        return self.bsn()



"""Provides data related to hardware."""

from mimesis.data import (CPU, CPU_CODENAMES, CPU_MODEL_CODES, GENERATION,
                          GRAPHICS, HDD_SSD, MANUFACTURERS, PHONE_MODELS,
                          RAM_SIZES, RAM_TYPES, RESOLUTIONS, SCREEN_SIZES)
from mimesis.providers.base import BaseDataProvider

__all__ = ['Hardware']


class Hardware(BaseDataProvider):
    """Class for generate data related to hardware."""

    def resolution(self) -> str:
        """Get a random screen resolution.

        :return: Resolution of screen.

        :Example:
            1280x720.
        """
        return self.random.choice(RESOLUTIONS)

    def screen_size(self) -> str:
        """Get a random size of screen in inch.

        :return: Screen size.

        :Example:
            13″.
        """
        return self.random.choice(SCREEN_SIZES)

    def cpu(self) -> str:
        """Get a random CPU name.

        :return: CPU name.

        :Example:
            Intel® Core i7.
        """
        return self.random.choice(CPU)

    def cpu_frequency(self) -> str:
        """Get a random frequency of CPU.

        :return: Frequency of CPU.

        :Example:
            4.0 GHz.
        """
        return '{}GHz'.format(
            self.random.uniform(
                a=1.5,
                b=4.3,
                precision=1,
            ),
        )

    def generation(self) -> str:
        """Get a random generation.

        :return: Generation of something.

        :Example:
             6th Generation.
        """
        return self.random.choice(GENERATION)

    def cpu_model_code(self) -> str:
        """Get a random CPU model.

        :return: CPU model.
        """
        return self.random.choice(CPU_MODEL_CODES)

    def cpu_codename(self) -> str:
        """Get a random CPU code name.

        :return: CPU code name.

        :Example:
            Cannonlake.
        """
        return self.random.choice(CPU_CODENAMES)

    def ram_type(self) -> str:
        """Get a random RAM type.

        :return: Type of RAM.

        :Example:
            DDR3.
        """
        return self.random.choice(RAM_TYPES)

    def ram_size(self) -> str:
        """Get a random size of RAM.

        :return: RAM size.

        :Example:
            16GB.
        """
        return self.random.choice(RAM_SIZES)

    def ssd_or_hdd(self) -> str:
        """Get a random value from list.

        :return: HDD or SSD.

        :Example:
            512GB SSD.
        """
        return self.random.choice(HDD_SSD)

    def graphics(self) -> str:
        """Get a random graphics.

        :return: Graphics.

        :Example:
            Intel® Iris™ Pro Graphics 6200.
        """
        return self.random.choice(GRAPHICS)

    def manufacturer(self) -> str:
        """Get a random manufacturer.

        :return: Manufacturer.

        :Example:
            Dell.
        """
        return self.random.choice(MANUFACTURERS)

    def phone_model(self) -> str:
        """Get a random phone model.

        :return: Phone model.

        :Example:
            Nokia Lumia 920.
        """
        return self.random.choice(PHONE_MODELS)



"""Provides data related to numbers."""

from typing import List, Union

from mimesis.providers.base import BaseDataProvider

__all__ = ['Numbers']


class Numbers(BaseDataProvider):
    """Class for generating numbers."""

    def floats(self, n: int = 2) -> List[float]:
        """Generate a list of random float numbers.

        :param n: Raise 10 to the 'n' power.
        :return: The list of floating-point numbers.
        """
        nums = [self.random.random()
                for _ in range(10 ** int(n))]
        return nums

    def integers(self, start: int = 0, end: int = 10,
                 length: int = 10) -> List[int]:
        """Generate a list of random integers.

        Integers can be negative or positive numbers.
        .. note: You can use both positive and negative numbers.

        :param start: Start.
        :param end: End.
        :param length: Length of list.
        :return: List of integers.

        :Example:
            [-20, -19, -18, -17]
        """
        return self.random.randints(
            length, start, end)

    @staticmethod
    def primes(start: int = 1, end: int = 999) -> List[int]:
        """Generate a list of prime numbers.

        :param start: First value of range.
        :param end: Last value of range.
        :return: A list of prime numbers from start to end.
        """
        # TODO: It should generate random primes with passed length.
        sieve_size = (end // 2 - 1) if end % 2 == 0 else (end // 2)
        sieve = [True] * sieve_size

        primes = []  # list of primes
        # add 2 to the list if it's in the given range
        if end >= 2:
            primes.append(2)
        for i in range(sieve_size):
            if sieve[i]:
                value_at_i = i * 2 + 3
                primes.append(value_at_i)
                for j in range(i, sieve_size, value_at_i):
                    sieve[j] = False

        chop_index = 0
        for i in range(len(primes)):
            if primes[i] >= start:
                chop_index = i
                break
        return primes[chop_index:]

    def digit(self, to_bin: bool = False) -> Union[str, int]:
        """Get a random digit.

        :param to_bin: If True then convert to binary.
        :return: Digit.

        :Example:
            4.
        """
        digit = self.random.randint(0, 9)

        if to_bin:
            return bin(digit)

        return digit

    def between(self, minimum: int = 1, maximum: int = 1000) -> int:
        """Generate a random number between minimum and maximum.

        :param minimum: Minimum of range.
        :param maximum: Maximum of range.
        :return: Number.
        """
        return self.random.randint(minimum, maximum)

    def rating(self, maximum: float = 5.0) -> float:
        """Generate a random rating for something.

        :param maximum: Maximum value (default is 5.0).
        :return: Rating.

        :Example:
            4.7
        """
        return self.random.uniform(0, maximum, 1)



"""Provides data related to text."""

from typing import List, Tuple

from mimesis.data import SAFE_COLORS
from mimesis.providers.base import BaseDataProvider
from mimesis.utils import pull

__all__ = ['Text']


class Text(BaseDataProvider):
    """Class for generating text data."""

    def __init__(self, *args, **kwargs):
        """Initialize attributes.

        :param locale: Current locale.
        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)
        self._data = pull('text.json', self.locale)

    def alphabet(self, lower_case: bool = False) -> list:
        """Get an alphabet for current locale.

        :param lower_case: Return alphabet in lower case.
        :return: Alphabet.
        """
        case = 'uppercase' if \
            not lower_case else 'lowercase'

        alpha = self._data['alphabet'].get(case)
        return alpha

    def level(self) -> str:
        """Generate a random level of danger or something else.

        :return: Level.

        :Example:
            critical.
        """
        levels = self._data['level']
        return self.random.choice(levels)

    def text(self, quantity: int = 5) -> str:
        """Generate the text.

        :param quantity: Quantity of sentences.
        :return: Text.
        """
        text = ''
        for _ in range(quantity):
            text += ' ' + self.random.choice(self._data['text'])
        return text.strip()

    def sentence(self) -> str:
        """Get a random sentence from text.

        :return: Sentence.
        """
        return self.text(quantity=1)

    def title(self) -> str:
        """Get a random title.

        :return: The title.
        """
        return self.text(quantity=1)

    def words(self, quantity: int = 5) -> List[str]:
        """Generate lis of the random words.

        :param quantity: Quantity of words. Default is 5.
        :return: Word list.

        :Example:
            [science, network, god, octopus, love]
        """
        words = self._data['words'].get('normal')
        words_list = [self.random.choice(words) for _ in range(quantity)]
        return words_list

    def word(self) -> str:
        """Get a random word.

        :return: Single word.

        :Example:
            Science.
        """
        return self.words(quantity=1)[0]

    def swear_word(self) -> str:
        """Get a random swear word.

        :return: Swear word.

        :Example:
            Damn.
        """
        bad_words = self._data['words'].get('bad')
        return self.random.choice(bad_words)

    def quote(self) -> str:
        """Get a random quote.

        :return: Quote from movie.

        :Example:
            "Bond... James Bond."
        """
        quotes = self._data['quotes']
        return self.random.choice(quotes)

    def color(self) -> str:
        """Get a random name of color.

        :return: Color name.

        :Example:
            Red.
        """
        colors = self._data['color']
        return self.random.choice(colors)

    @staticmethod
    def _hex_to_rgb(color: str) -> Tuple[int, ...]:
        """Convert hex color to RGB format.

        :param color: Hex color.
        :return: RGB tuple.
        """
        if color.startswith('#'):
            color = color.lstrip('#')
        return tuple(int(color[i:i + 2], 16) for i in (0, 2, 4))

    def hex_color(self, safe: bool = False) -> str:
        """Generate a random hex color.

        :param safe: Get safe Flat UI hex color.
        :return: Hex color code.

        :Example:
            #d8346b
        """
        if safe:
            return self.random.choice(SAFE_COLORS)

        return '#{:06x}'.format(
            self.random.randint(0x000000, 0xffffff))

    def rgb_color(self, safe: bool = False) -> Tuple[int, ...]:
        """Generate a random rgb color tuple.

        :param safe: Get safe RGB tuple.
        :return: RGB tuple.

        :Example:
            (252, 85, 32)
        """
        color = self.hex_color(safe)
        return self._hex_to_rgb(color)

    def answer(self) -> str:
        """Get a random answer in current language.

        :return: An answer.

        :Example:
            No
        """
        answers = self._data['answers']
        return self.random.choice(answers)



"""File data provider."""

import re
from typing import Optional

from mimesis.data import EXTENSIONS, MIME_TYPES
from mimesis.enums import FileType, MimeType
from mimesis.providers.base import BaseDataProvider
from mimesis.providers.text import Text

__all__ = ['File']


class File(BaseDataProvider):
    """Class for generate data related to files."""

    def __init__(self, *args, **kwargs):
        """Initialize attributes.

        :param args: Arguments.
        :param kwargs: Keyword arguments.
        """
        super().__init__(*args, **kwargs)
        self.__text = Text('en', seed=self.seed)

    def __sub(self, string: str = '') -> str:
        """Replace spaces in string.

        :param string: String.
        :return: String without spaces.
        """
        replacer = self.random.choice(['_', '-'])
        return re.sub('\s+', replacer, string.strip())

    def extension(self, file_type: Optional[FileType] = None) -> str:
        """Get a random file extension from list.

        :param file_type: Enum object FileType.
        :return: Extension of the file.

        :Example:
            .py
        """
        key = self._validate_enum(item=file_type, enum=FileType)
        extensions = EXTENSIONS[key]
        return self.random.choice(extensions)

    def mime_type(self, type_: Optional[MimeType] = None) -> str:
        """Get a random mime type from list.

        :param type_: Enum object MimeType.
        :return: Mime type.
        """
        key = self._validate_enum(item=type_, enum=MimeType)
        types = MIME_TYPES[key]
        return self.random.choice(types)

    def size(self, minimum: int = 1, maximum: int = 100) -> str:
        """Get size of file.

        :param minimum: Maximum value.
        :param maximum: Minimum value.
        :return: Size of file.

        :Example:
            56 kB
        """
        num = self.random.randint(minimum, maximum)
        unit = self.random.choice(
            ['bytes', 'kB', 'MB', 'GB', 'TB'])

        return '{num} {unit}'.format(
            num=num,
            unit=unit,
        )

    def file_name(self, file_type: Optional[FileType] = None) -> str:
        """Get a random file name with some extension.

        :param file_type: Enum object FileType
        :return: File name.

        :Example:
            legislative.txt
        """
        name = self.__text.word()
        ext = self.extension(file_type)

        return '{name}{ext}'.format(
            name=self.__sub(name),
            ext=ext,
        )



"""Provides all the data related to the hardware."""

RESOLUTIONS = [
    '1152x768',
    '1280x854',
    '1440x960',
    '2880x1920',
    '1024x768',
    '1152x864',
    '1280x960',
    '1400x1050',
    '1600x1200',
    '2048x1536',
    '3200x2400',
    '1280x768',
    '1280x1024',
    '2560x2048',
    '1280x720',
    '1365x768',
    '1600x900',
    '1920x1080',
    '1280x800',
    '1440x900',
    '1680x1050',
    '1920x1200',
    '2560x1600',
]

SCREEN_SIZES = [
    '14″',
    '12.1″',
    '12″',
    '14.4″',
    '15″',
    '15.7″',
    '13.3″',
    '13″',
    '17″',
    '15.4″',
    '14.1″',
]

CPU = [
    'AMD Ryzen 7 1800X',
    'AMD Ryzen 7 1700',
    'AMD Ryzen™ Threadripper™',
    'Intel® Core i3',
    'Intel® Core i5',
    'Intel® Core i7',
    'Intel® Core i9',
]

RAM_TYPES = [
    'SDRAM',
    'DDR',
    'DDR2',
    'DDR3',
    'DDR4',
]

RAM_SIZES = [
    '4GB',
    '8GB',
    '16GB',
    '32GB',
    '64GB',
]

GENERATION = [
    '2nd Generation',
    '3rd Generation',
    '4th Generation',
    '5th Generation',
    '6th Generation',
    '7th Generation',
]

CPU_MODEL_CODES = [
    '7500U',
    '7Y75',
    '7Y30',
    '7200U',
    '7500U',
    '6600K',
    '6700T',
    '6700K',
    '6600T',
    '6300T',
    '6100H',
    '6820HK',
    '6920HQ',
    '6700HQ',
    '6600U',
    '6300U',
    '6100U',
    '5775C',
    '5675C',
    '5775R',
    '5675R',
    '5575R',
    '5350H',
    '5950HQ',
    '5850HQ',
    '5750HQ',
    '5700HQ',
    '5650U',
]

CPU_CODENAMES = [
    'Ivytown',
    'Haswell',
    'Fortville',
    'Devil\'s Canyon',
    'Valley Island',
    'Broadwell',
    'Bay Trail',
    'Skylake',
    'Orchid Island',
    'Bear Ridge',
    'Cannonlake',
]

HDD_SSD = [
    '64GB SSD',
    '128GB SSD',
    '256GB SDD',
    '512GB SSD',
    '1024GB SSD',
    '256GB HDD',
    '256GB HDD(7200 RPM)',
    '256GB HDD(5400 RPM)',
    '512GB HDD',
    '512GB HDD(7200 RPM)',
    '1TB HDD',
    '1TB HDD(7200 RPM)',
    '1TB HDD + 64GB SSD',
    '2TB HDD(7200 RPM)',
    '512 GB HDD + 32GB SSD',
    '1TB HDD(7200 RPM) + 32GB SSD',
]

GRAPHICS = [
    'Intel® HD Graphics 620',
    'Intel® HD Graphics 615',
    'Intel® Iris™ Pro Graphics 580',
    'Intel® Iris™ Graphics 550',
    'Intel® HD Graphics 520',
    'Intel® Iris™ Pro Graphics 6200',
    'Intel® Iris™ Graphics 6100',
    'Intel® HD Graphics 6000',
    'Intel® HD Graphics 5300 ',
    'Intel® Iris™ Pro Graphics 5200',
    'Intel® Iris™ Graphics 5100',
    'Intel® HD Graphics 5500',
    'Intel® HD Graphics 5000',
    'Intel® HD Graphics 4600',
    'Intel® HD Graphics 4400',
    'Intel® HD Graphics 4000',
    'Intel® HD Graphics 3000',
    'NVIDIA GeForce GTX 1080',
    'NVIDIA GeForce GTX 1070',
    'NVIDIA GeForce GTX 1080',
    'NVIDIA GeForce GTX 980',
    'NVIDIA GeForce GTX 1070',
    'NVIDIA GeForce GTX 980M',
    'NVIDIA GeForce GTX 980',
    'NVIDIA GeForce GTX 970M',
    'NVIDIA GeForce GTX 880M',
    'AMD Radeon R9 M395X',
    'AMD Radeon R9 M485X',
    'AMD Radeon R9 M395',
]

MANUFACTURERS = [
    'Apple',
    'Acer',
    'Dell',
    'ASUS',
    'VAIO',
    'Lenovo',
    'HP',
    'Toshiba',
    'Sony',
    'Samsung',
    'Fujitsu',
    'Apple',
]

PHONE_MODELS = [
    'Apple iPhone 4',
    'Apple iPhone 4S',
    'Apple iPhone 5',
    'Apple iPhone 5s',
    'Apple iPhone 6',
    'Apple iPhone 6 Plus',
    'Apple iPhone 6s',
    'Apple iPhone 6s Plus',
    'Apple iPhone 7',
    'Apple iPhone 7 Plus',
    'Apple iPhone SE',
    'BlackBerry 10 Classic',
    'BlackBerry 10 Leap',
    'BlackBerry 10 P\'9982',
    'BlackBerry 10 Passport',
    'BlackBerry 10 Q10',
    'BlackBerry 10 Q5',
    'BlackBerry 10 Z10',
    'BlackBerry 10 Z3',
    'BlackBerry 10 Z30',
    'Nexus 5',
    'Nexus 5X',
    'Nexus 6',
    'Nexus 6P',
    'Nexus 7',
    'Nokia Lumia 1020',
    'Nokia Lumia 1320',
    'Nokia Lumia 1520',
    'Nokia Lumia 2520',
    'Nokia Lumia 505',
    'Nokia Lumia 510',
    'Nokia Lumia 520',
    'Nokia Lumia 525',
    'Nokia Lumia 530',
    'Nokia Lumia 610',
    'Nokia Lumia 620',
    'Nokia Lumia 625',
    'Nokia Lumia 630',
    'Nokia Lumia 635',
    'Nokia Lumia 636',
    'Nokia Lumia 638',
    'Nokia Lumia 710',
    'Nokia Lumia 720',
    'Nokia Lumia 730',
    'Nokia Lumia 735',
    'Nokia Lumia 800',
    'Nokia Lumia 810',
    'Nokia Lumia 820',
    'Nokia Lumia 822',
    'Nokia Lumia 830',
    'Nokia Lumia 900',
    'Nokia Lumia 920',
    'Nokia Lumia 925',
    'Nokia Lumia 928',
    'Nokia Lumia 930',
    'Nokia Lumia ICON',
    'Samsung GT-B7330',
    'Samsung GT-C3520',
    'Samsung GT-E1150i',
    'Samsung Galaxy',
    'Samsung Galaxy',
    'Samsung Galaxy 5',
    'Samsung Galaxy Ace 2',
    'Samsung Galaxy Ace 3',
    'Samsung Galaxy Ace 4',
    'Samsung Galaxy Active Neo',
    'Samsung Galaxy Alpha',
    'Samsung Galaxy Beam i8530',
    'Samsung Galaxy Chat',
    'Samsung Galaxy Core',
    'Samsung Galaxy Core Advance',
    'Samsung Galaxy Core LTE',
    'Samsung Galaxy Core Prime',
    'Samsung Galaxy Duos',
    'Samsung Galaxy E5',
    'Samsung Galaxy E7',
    'Samsung Galaxy Grand Prime',
    'Samsung Galaxy J',
    'Samsung Galaxy J1 Ace Neo',
    'Samsung Galaxy J1 Ace VE',
    'Samsung Galaxy J1 Mini',
    'Samsung Galaxy J5',
    'Samsung Galaxy J7 (2016)',
    'Samsung Galaxy Mega',
    'Samsung Galaxy Mega 2',
    'Samsung Galaxy Note 3',
    'Samsung Galaxy Note 3 Neo',
    'Samsung Galaxy Note 4',
    'Samsung Galaxy Note 5',
    'Samsung Galaxy Note 7',
    'Samsung Galaxy Note Edge',
    'Samsung Galaxy Note II',
    'Samsung Galaxy Note series',
    'Samsung Galaxy Pocket',
    'Samsung Galaxy Pocket 2',
    'Samsung Galaxy Pocket Duos',
    'Samsung Galaxy Pocket Neo',
    'Samsung Galaxy Pocket Plus',
    'Samsung Galaxy Portal',
    'Samsung Galaxy Prevail',
    'Samsung Galaxy R',
    'Samsung Galaxy Round',
    'Samsung Galaxy S',
    'Samsung Galaxy S 4G LTE',
    'Samsung Galaxy S Advance',
    'Samsung Galaxy S II',
    'Samsung Galaxy S III Mini',
    'Samsung Galaxy S III Neo',
    'Samsung Galaxy S Plus',
    'Samsung Galaxy S series',
    'Samsung Galaxy S6',
    'Samsung Galaxy Spica',
    'Samsung Galaxy Star',
    'Samsung Galaxy Tab series',
    'Samsung Galaxy Trend 2 Lite',
    'Samsung Galaxy W',
    'Samsung Galaxy Y',
    'Samsung Galaxy Young',
    'Samsung Galaxy Young 2',
    'Samsung Gravity series',
    'Samsung SGH-G600',
    'Samsung SGH-G800',
    'Samsung SGH-G810',
    'Sony Xperia E5',
    'Sony Xperia X',
    'Sony Xperia XA',
    'Sony Xperia XA Dual',
    'Sony Xperia XZ',
    'Sony Xperia Z5 Premium',
]



"""Provides data related to transports."""

from mimesis.data import AIRPLANES, CARS, TRUCKS, VR_CODES, VRC_BY_LOCALES
from mimesis.providers.base import BaseDataProvider

__all__ = ['Transport']


class Transport(BaseDataProvider):
    """Class for generating data related to transports."""

    def __init__(self, *args, **kwargs) -> None:
        """Initialize attributes.

        :param locale: Current locale.
        :param seed: Seed.
        """
        super().__init__(*args, **kwargs)

    def truck(self, model_mask: str = '#### @@') -> str:
        """Generate a truck model.

        :param model_mask: Mask of truck model. Here '@' is a
            placeholder of characters and '#' is a placeholder of digits.
        :return: Dummy truck model.

        :Example:
            Caledon-966O.
        """
        return '{}-{}'.format(
            self.random.choice(TRUCKS),
            self.random.custom_code(model_mask),
        )

    def car(self) -> str:
        """Get a random vehicle.

        :return: A vehicle.

        :Example:
            Tesla Model S.
        """
        return self.random.choice(CARS)

    def airplane(self, model_mask: str = '###') -> str:
        """Generate a dummy airplane model.

        :param model_mask: Mask of truck model. Here '@' is a
            placeholder of characters and '#' is a placeholder of digits.
        :return: Airplane model.

        :Example:
            Boeing 727.
        """
        model = self.random.custom_code(mask=model_mask)
        plane = self.random.choice(AIRPLANES)
        return '{} {}'.format(plane, model)

    def vehicle_registration_code(self, allow_random: bool = True) -> str:
        """Get vehicle registration code of country.

        :param allow_random: If False return only code
            for current locale (country).
        :return: Vehicle registration code.
        """
        if not allow_random:
            return VRC_BY_LOCALES[self.locale]

        return self.random.choice(VR_CODES)



"""Provides data related to units."""

from typing import Optional

from mimesis.data import SI_PREFIXES, SI_PREFIXES_SYM
from mimesis.enums import PrefixSign, UnitName
from mimesis.providers.base import BaseDataProvider

__all__ = ['UnitSystem']


class UnitSystem(BaseDataProvider):
    """Class for generating data related to units."""

    def unit(self, name: Optional[UnitName] = None, symbol=False):
        """Get unit name.

        :param name: Enum object UnitName.
        :param symbol: Return only symbol
        :return: Unit.
        """
        result = self._validate_enum(item=name, enum=UnitName)

        if symbol:
            return result[1]
        return result[0]

    def prefix(self, sign: Optional[PrefixSign] = None,
               symbol: bool = False) -> str:
        """Get a random prefix for the International System of Units.

        :param sign: Sing of number.
        :param symbol: Return symbol of prefix.
        :return: Prefix for SI.
        :raises NonEnumerableError: if sign is not supported.

        :Example:
            mega
        """
        prefixes = SI_PREFIXES_SYM if \
            symbol else SI_PREFIXES

        key = self._validate_enum(item=sign, enum=PrefixSign)
        return self.random.choice(prefixes[key])



"""Provides all the data related to the files."""

EXTENSIONS = {
    'source': [
        '.a',
        '.asm',
        '.asp',
        '.awk',
        '.c',
        '.class',
        '.cpp',
        '.pl',
        '.js',
        '.java',
        '.clj',
        '.py',
        '.rb',
        '.hs',
        '.erl',
        '.rs',
        '.swift',
        '.html',
        '.json',
        '.xml',
        '.css',
        '.php',
        '.jl',
        '.r',
        '.cs',
        'd',
        '.lisp',
        '.cl',
        '.go',
        '.h',
        '.scala',
        '.sc',
        '.ts',
        '.sql',
    ],
    'text': [
        '.doc',
        '.docx',
        '.log',
        '.rtf',
        '.md',
        '.pdf',
        '.odt',
        '.txt',
    ],
    'data': [
        '.csv',
        '.dat',
        '.ged',
        '.pps',
        '.ppt',
        '.pptx',
    ],
    'audio': [
        '.flac',
        '.mp3',
        '.m3u',
        '.m4a',
        '.wav',
        '.wma',
    ],
    'video': [
        '.3gp',
        '.mp4',
        '.abi',
        '.m4v',
        '.mov',
        '.mpg',
        '.wmv',
    ],
    'image': [
        '.bmp',
        '.jpg',
        '.jpeg',
        '.png',
        '.svg',
    ],
    'executable': [
        '.apk',
        '.app',
        '.bat',
        '.jar',
        '.com',
        '.exe',
    ],
    'compressed': [
        '.7z',
        '.war',
        '.zip',
        '.tar.gz',
        '.tar.xz',
        '.rar',
    ],
}

MIME_TYPES = {
    'application': [
        'application/1d-interleaved-parityfec',
        'application/3gpdash-qoe-report+xml',
        'application/3gpp-ims+xml',
        'application/A2L',
        'application/activemessage',
        'application/alto-costmap+json',
        'application/alto-costmapfilter+json',
        'application/alto-directory+json',
        'application/alto-endpointcost+json',
        'application/alto-endpointcostparams+json',
        'application/alto-endpointprop+json',
        'application/alto-endpointpropparams+json',
        'application/alto-error+json',
        'application/alto-networkmap+json',
        'application/alto-networkmapfilter+json',
        'application/AML',
        'application/andrew-inset',
        'application/applefile',
        'application/ATF',
        'application/ATFX',
        'application/atom+xml',
        'application/atomcat+xml',
        'application/atomdeleted+xml',
        'application/atomicmail',
        'application/atomsvc+xml',
        'application/ATXML',
        'application/auth-policy+xml',
        'application/bacnet-xdd+zip',
        'application/batch-SMTP',
        'application/beep+xml',
        'application/calendar+json',
        'application/calendar+xml',
        'application/call-completion',
        'application/cals-1840',
        'application/cbor',
        'application/ccmp+xml',
        'application/ccxml+xml',
        'application/CDFX+XML',
        'application/cdmi-capability',
        'application/cdmi-container',
        'application/cdmi-domain',
        'application/cdmi-object',
        'application/cdmi-queue',
        'application/CEA',
        'application/cea-2018+xml',
        'application/cellml+xml',
        'application/cfw',
        'application/cms',
        'application/cnrp+xml',
        'application/coap-group+json',
        'application/commonground',
        'application/conference-info+xml',
        'application/cpl+xml',
        'application/csrattrs',
        'application/csta+xml',
        'application/CSTAdata+xml',
        'application/cybercash',
        'application/dash+xml',
        'application/dashdelta',
        'application/davmount+xml',
        'application/dca-rft',
        'application/DCD',
        'application/dec-dx',
        'application/dialog-info+xml',
        'application/dicom',
        'application/DII',
        'application/DIT',
        'application/dns',
        'application/dskpp+xml',
        'application/dssc+der',
        'application/dssc+xml',
        'application/dvcs',
        'application/ecmascript',
        'application/EDI-consent',
        'application/EDI-X12',
        'application/EDIFACT',
        'application/emma+xml',
        'application/emotionml+xml',
        'application/encaprtp',
        'application/epp+xml',
        'application/epub+zip',
        'application/eshop',
        'application/example',
        'application/exi',
        'application/fastinfoset',
        'application/fastsoap',
        'application/fdt+xml',
        'application/fits',
        'application/font-sfnt',
        'application/font-tdpfr',
        'application/font-woff',
        'application/framework-attributes+xml',
        'application/gzip',
        'application/H224',
        'application/held+xml',
        'application/http',
        'application/hyperstudio',
        'application/ibe-key-request+xml',
        'application/ibe-pkg-reply+xml',
        'application/ibe-pp-data',
        'application/iges',
        'application/im-iscomposing+xml',
        'application/index',
        'application/index.cmd',
        'application/index.obj',
        'application/index.response',
        'application/index.vnd',
        'application/inkml+xml',
        'application/iotp',
        'application/ipfix',
        'application/ipp',
        'application/isup',
        'application/its+xml',
        'application/javascript',
        'application/jose',
        'application/jose+json',
        'application/jrd+json',
        'application/json',
        'application/json-patch+json',
        'application/json-seq',
        'application/jwk-set+json',
        'application/jwk+json',
        'application/jwt',
        'application/kpml-request+xml',
        'application/kpml-response+xml',
        'application/ld+json',
        'application/link-format',
        'application/load-control+xml',
        'application/lost+xml',
        'application/lostsync+xml',
        'application/LXF',
        'application/mac-binhex40',
        'application/macwriteii',
        'application/mads+xml',
        'application/marc',
        'application/marcxml+xml',
        'application/mathematica',
        'application/mathml-content+xml',
        'application/mathml-presentation+xml',
        'application/mathml+xml',
        'application/mbms-associated-procedure-description+xml',
        'application/mbms-deregister+xml',
        'application/mbms-envelope+xml',
        'application/mbms-msk-response+xml',
        'application/mbms-msk+xml',
        'application/mbms-protection-description+xml',
        'application/mbms-reception-report+xml',
        'application/mbms-register-response+xml',
        'application/mbms-register+xml',
        'application/mbms-schedule+xml',
        'application/mbms-user-service-description+xml',
        'application/mbox',
        'application/media_control+xml',
        'application/media-policy-dataset+xml',
        'application/mediaservercontrol+xml',
        'application/merge-patch+json',
        'application/metalink4+xml',
        'application/mets+xml',
        'application/MF4',
        'application/mikey',
        'application/mods+xml',
        'application/moss-keys',
        'application/moss-signature',
        'application/mosskey-data',
        'application/mosskey-request',
        'application/mp21',
        'application/mp4',
        'application/mpeg4-generic',
        'application/mpeg4-iod',
        'application/mpeg4-iod-xmt',
        'application/mrb-consumer+xml',
        'application/mrb-publish+xml',
        'application/msc-ivr+xml',
        'application/msc-mixer+xml',
        'application/msword',
        'application/mxf',
        'application/nasdata',
        'application/news-checkgroups',
        'application/news-groupinfo',
        'application/news-transmission',
        'application/nlsml+xml',
        'application/nss',
        'application/ocsp-request',
        'application/ocsp-response',
        'application/octet-stream',
        'application/oda',
        'application/ODX',
        'application/oebps-package+xml',
        'application/ogg',
        'application/oxps',
        'application/p2p-overlay+xml',
        'application/parityfec',
        'application/patch-ops-error+xml',
        'application/pdf',
        'application/PDX',
        'application/pgp-encrypted',
        'application/pgp-keys',
        'application/pgp-signature',
        'application/pidf-diff+xml',
        'application/pidf+xml',
        'application/pkcs10',
        'application/pkcs7-mime',
        'application/pkcs7-signature',
        'application/pkcs8',
        'application/pkix-attr-cert',
        'application/pkix-cert',
        'application/pkix-crl',
        'application/pkix-pkipath',
        'application/pkixcmp',
        'application/pls+xml',
        'application/poc-settings+xml',
        'application/postscript',
        'application/provenance+xml',
        'application/prs.alvestrand.titrax-sheet',
        'application/prs.cww',
        'application/prs.hpub+zip',
        'application/prs.nprend',
        'application/prs.plucker',
        'application/prs.rdf-xml-crypt',
        'application/prs.xsf+xml',
        'application/pskc+xml',
        'application/qsig',
        'application/raptorfec',
        'application/rdap+json',
        'application/rdf+xml',
        'application/reginfo+xml',
        'application/relax-ng-compact-syntax',
        'application/remote-printing',
        'application/reputon+json',
        'application/resource-lists-diff+xml',
        'application/resource-lists+xml',
        'application/riscos',
        'application/rlmi+xml',
        'application/rls-services+xml',
        'application/rpki-ghostbusters',
        'application/rpki-manifest',
        'application/rpki-roa',
        'application/rpki-updown',
        'application/rtf',
        'application/rtploopback',
        'application/rtx',
        'application/samlassertion+xml',
        'application/samlmetadata+xml',
        'application/sbml+xml',
        'application/scaip+xml',
        'application/scvp-cv-request',
        'application/scvp-cv-response',
        'application/scvp-vp-request',
        'application/scvp-vp-response',
        'application/sdp',
        'application/sep-exi',
        'application/sep+xml',
        'application/session-info',
        'application/set-payment',
        'application/set-payment-initiation',
        'application/set-registration',
        'application/set-registration-initiation',
        'application/sgml',
        'application/sgml-open-catalog',
        'application/shf+xml',
        'application/sieve',
        'application/simple-filter+xml',
        'application/simple-message-summary',
        'application/simpleSymbolContainer',
        'application/slate',
        'application/smil - OBSOLETED in favor of application/smil+xml',
        'application/smil+xml',
        'application/smpte336m',
        'application/soap+fastinfoset',
        'application/soap+xml',
        'application/sparql-query',
        'application/sparql-results+xml',
        'application/spirits-event+xml',
        'application/sql',
        'application/srgs',
        'application/srgs+xml',
        'application/sru+xml',
        'application/ssml+xml',
        'application/tamp-apex-update',
        'application/tamp-apex-update-confirm',
        'application/tamp-community-update',
        'application/tamp-community-update-confirm',
        'application/tamp-error',
        'application/tamp-sequence-adjust',
        'application/tamp-sequence-adjust-confirm',
        'application/tamp-status-query',
        'application/tamp-status-response',
        'application/tamp-update',
        'application/tamp-update-confirm',
        'application/tei+xml',
        'application/thraud+xml',
        'application/timestamp-query',
        'application/timestamp-reply',
        'application/timestamped-data',
        'application/ttml+xml',
        'application/tve-trigger',
        'application/ulpfec',
        'application/urc-grpsheet+xml',
        'application/urc-ressheet+xml',
        'application/urc-targetdesc+xml',
        'application/urc-uisocketdesc+xml',
        'application/vcard+json',
        'application/vcard+xml',
        'application/vemmi',
        'application/vnd.3gpp.bsf+xml',
        'application/vnd.3gpp.pic-bw-large',
        'application/vnd.3gpp.pic-bw-small',
        'application/vnd.3gpp.pic-bw-var',
        'application/vnd.3gpp.sms',
        'application/vnd.3gpp2.bcmcsinfo+xml',
        'application/vnd.3gpp2.sms',
        'application/vnd.3gpp2.tcap',
        'application/vnd.3M.Post-it-Notes',
        'application/vnd.accpac.simply.aso',
        'application/vnd.accpac.simply.imp',
        'application/vnd.acucobol',
        'application/vnd.acucorp',
        'application/vnd.adobe.flash.movie',
        'application/vnd.adobe.formscentral.fcdt',
        'application/vnd.adobe.fxp',
        'application/vnd.adobe.partial-upload',
        'application/vnd.adobe.xdp+xml',
        'application/vnd.adobe.xfdf',
        'application/vnd.aether.imp',
        'application/vnd.ah-barcode',
        'application/vnd.ahead.space',
        'application/vnd.airzip.filesecure.azf',
        'application/vnd.airzip.filesecure.azs',
        'application/vnd.americandynamics.acc',
        'application/vnd.amiga.ami',
        'application/vnd.amundsen.maze+xml',
        'application/vnd.anser-web-certificate-issue-initiation',
        'application/vnd.antix.game-component',
        'application/vnd.apache.thrift.binary',
        'application/vnd.apache.thrift.compact',
        'application/vnd.apache.thrift.json',
        'application/vnd.api+json',
        'application/vnd.apple.installer+xml',
        'application/vnd.apple.mpegurl',
        'application/vnd.aristanetworks.swi',
        'application/vnd.artsquare',
        'application/vnd.astraea-software.iota',
        'application/vnd.audiograph',
        'application/vnd.autopackage',
        'application/vnd.avistar+xml',
        'application/vnd.balsamiq.bmml+xml',
        'application/vnd.bekitzur-stech+json',
        'application/vnd.blueice.multipass',
        'application/vnd.bluetooth.ep.oob',
        'application/vnd.bluetooth.le.oob',
        'application/vnd.bmi',
        'application/vnd.businessobjects',
        'application/vnd.cab-jscript',
        'application/vnd.canon-cpdl',
        'application/vnd.canon-lips',
        'application/vnd.cendio.thinlinc.clientconf',
        'application/vnd.century-systems.tcp_stream',
        'application/vnd.chemdraw+xml',
        'application/vnd.chipnuts.karaoke-mmd',
        'application/vnd.cinderella',
        'application/vnd.cirpack.isdn-ext',
        'application/vnd.citationstyles.style+xml',
        'application/vnd.claymore',
        'application/vnd.cloanto.rp9',
        'application/vnd.clonk.c4group',
        'application/vnd.cluetrust.cartomobile-config',
        'application/vnd.cluetrust.cartomobile-config-pkg',
        'application/vnd.coffeescript',
        'application/vnd.collection.doc+json',
        'application/vnd.collection.next+json',
        'application/vnd.collection+json',
        'application/vnd.commerce-battelle',
        'application/vnd.commonspace',
        'application/vnd.contact.cmsg',
        'application/vnd.cosmocaller',
        'application/vnd.crick.clicker',
        'application/vnd.crick.clicker.keyboard',
        'application/vnd.crick.clicker.palette',
        'application/vnd.crick.clicker.template',
        'application/vnd.crick.clicker.wordbank',
        'application/vnd.criticaltools.wbs+xml',
        'application/vnd.ctc-posml',
        'application/vnd.ctct.ws+xml',
        'application/vnd.cups-pdf',
        'application/vnd.cups-postscript',
        'application/vnd.cups-ppd',
        'application/vnd.cups-raster',
        'application/vnd.cups-raw',
        'application/vnd.curl',
        'application/vnd.cyan.dean.root+xml',
        'application/vnd.cybank',
        'application/vnd.dart',
        'application/vnd.data-vision.rdz',
        'application/vnd.debian.binary-package',
        'application/vnd.dece.data',
        'application/vnd.dece.ttml+xml',
        'application/vnd.dece.unspecified',
        'application/vnd.dece.zip',
        'application/vnd.denovo.fcselayout-link',
        'application/vnd.desmume.movie',
        'application/vnd.dir-bi.plate-dl-nosuffix',
        'application/vnd.dm.delegation+xml',
        'application/vnd.dna',
        'application/vnd.document+json',
        'application/vnd.dolby.mobile.1',
        'application/vnd.dolby.mobile.2',
        'application/vnd.doremir.scorecloud-binary-document',
        'application/vnd.dpgraph',
        'application/vnd.dreamfactory',
        'application/vnd.dtg.local',
        'application/vnd.dtg.local.flash',
        'application/vnd.dtg.local.html',
        'application/vnd.dvb.ait',
        'application/vnd.dvb.dvbj',
        'application/vnd.dvb.esgcontainer',
        'application/vnd.dvb.ipdcdftnotifaccess',
        'application/vnd.dvb.ipdcesgaccess',
        'application/vnd.dvb.ipdcesgaccess2',
        'application/vnd.dvb.ipdcesgpdd',
        'application/vnd.dvb.ipdcroaming',
        'application/vnd.dvb.iptv.alfec-base',
        'application/vnd.dvb.iptv.alfec-enhancement',
        'application/vnd.dvb.notif-aggregate-root+xml',
        'application/vnd.dvb.notif-container+xml',
        'application/vnd.dvb.notif-generic+xml',
        'application/vnd.dvb.notif-ia-msglist+xml',
        'application/vnd.dvb.notif-ia-registration-request+xml',
        'application/vnd.dvb.notif-ia-registration-response+xml',
        'application/vnd.dvb.notif-init+xml',
        'application/vnd.dvb.pfr',
        'application/vnd.dvb.service',
        'application/vnd.dxr',
        'application/vnd.dynageo',
        'application/vnd.dzr',
        'application/vnd.easykaraoke.cdgdownload',
        'application/vnd.ecdis-update',
        'application/vnd.ecowin.chart',
        'application/vnd.ecowin.filerequest',
        'application/vnd.ecowin.fileupdate',
        'application/vnd.ecowin.series',
        'application/vnd.ecowin.seriesrequest',
        'application/vnd.ecowin.seriesupdate',
        'application/vnd.emclient.accessrequest+xml',
        'application/vnd.enliven',
        'application/vnd.enphase.envoy',
        'application/vnd.eprints.data+xml',
        'application/vnd.epson.esf',
        'application/vnd.epson.msf',
        'application/vnd.epson.quickanime',
        'application/vnd.epson.salt',
        'application/vnd.epson.ssf',
        'application/vnd.ericsson.quickcall',
        'application/vnd.eszigno3+xml',
        'application/vnd.etsi.aoc+xml',
        'application/vnd.etsi.asic-e+zip',
        'application/vnd.etsi.asic-s+zip',
        'application/vnd.etsi.cug+xml',
        'application/vnd.etsi.iptvcommand+xml',
        'application/vnd.etsi.iptvdiscovery+xml',
        'application/vnd.etsi.iptvprofile+xml',
        'application/vnd.etsi.iptvsad-bc+xml',
        'application/vnd.etsi.iptvsad-cod+xml',
        'application/vnd.etsi.iptvsad-npvr+xml',
        'application/vnd.etsi.iptvservice+xml',
        'application/vnd.etsi.iptvsync+xml',
        'application/vnd.etsi.iptvueprofile+xml',
        'application/vnd.etsi.mcid+xml',
        'application/vnd.etsi.mheg5',
        'application/vnd.etsi.overload-control-policy-dataset+xml',
        'application/vnd.etsi.pstn+xml',
        'application/vnd.etsi.sci+xml',
        'application/vnd.etsi.simservs+xml',
        'application/vnd.etsi.timestamp-token',
        'application/vnd.etsi.tsl.der',
        'application/vnd.etsi.tsl+xml',
        'application/vnd.eudora.data',
        'application/vnd.ezpix-album',
        'application/vnd.ezpix-package',
        'application/vnd.f-secure.mobile',
        'application/vnd.fdf',
        'application/vnd.fdsn.mseed',
        'application/vnd.fdsn.seed',
        'application/vnd.ffsns',
        'application/vnd.fints',
        'application/vnd.FloGraphIt',
        'application/vnd.fluxtime.clip',
        'application/vnd.font-fontforge-sfd',
        'application/vnd.framemaker',
        'application/vnd.frogans.fnc',
        'application/vnd.frogans.ltf',
        'application/vnd.fsc.weblaunch',
        'application/vnd.fujitsu.oasys',
        'application/vnd.fujitsu.oasys2',
        'application/vnd.fujitsu.oasys3',
        'application/vnd.fujitsu.oasysgp',
        'application/vnd.fujitsu.oasysprs',
        'application/vnd.fujixerox.ART-EX',
        'application/vnd.fujixerox.ART4',
        'application/vnd.fujixerox.ddd',
        'application/vnd.fujixerox.docuworks',
        'application/vnd.fujixerox.docuworks.binder',
        'application/vnd.fujixerox.docuworks.container',
        'application/vnd.fujixerox.HBPL',
        'application/vnd.fut-misnet',
        'application/vnd.fuzzysheet',
        'application/vnd.genomatix.tuxedo',
        'application/vnd.geo+json',
        'application/vnd.geocube+xml - OBSOLETED by request',
        'application/vnd.geogebra.file',
        'application/vnd.geogebra.tool',
        'application/vnd.geometry-explorer',
        'application/vnd.geonext',
        'application/vnd.geoplan',
        'application/vnd.geospace',
        'application/vnd.gerber',
        'application/vnd.globalplatform.card-content-mgt',
        'application/vnd.globalplatform.card-content-mgt-response',
        'application/vnd.gmx - DEPRECATED',
        'application/vnd.google-earth.kml+xml',
        'application/vnd.google-earth.kmz',
        'application/vnd.gov.sk.e-form+xml',
        'application/vnd.gov.sk.e-form+zip',
        'application/vnd.gov.sk.xmldatacontainer+xml',
        'application/vnd.grafeq',
        'application/vnd.gridmp',
        'application/vnd.groove-account',
        'application/vnd.groove-help',
        'application/vnd.groove-identity-message',
        'application/vnd.groove-injector',
        'application/vnd.groove-tool-message',
        'application/vnd.groove-tool-template',
        'application/vnd.groove-vcard',
        'application/vnd.hal+json',
        'application/vnd.hal+xml',
        'application/vnd.HandHeld-Entertainment+xml',
        'application/vnd.hbci',
        'application/vnd.hcl-bireports',
        'application/vnd.heroku+json',
        'application/vnd.hhe.lesson-player',
        'application/vnd.hp-HPGL',
        'application/vnd.hp-hpid',
        'application/vnd.hp-hps',
        'application/vnd.hp-jlyt',
        'application/vnd.hp-PCL',
        'application/vnd.hp-PCLXL',
        'application/vnd.httphone',
        'application/vnd.hydrostatix.sof-data',
        'application/vnd.hzn-3d-crossword',
        'application/vnd.ibm.afplinedata',
        'application/vnd.ibm.electronic-media',
        'application/vnd.ibm.MiniPay',
        'application/vnd.ibm.modcap',
        'application/vnd.ibm.rights-management',
        'application/vnd.ibm.secure-container',
        'application/vnd.iccprofile',
        'application/vnd.ieee.1905',
        'application/vnd.igloader',
        'application/vnd.immervision-ivp',
        'application/vnd.immervision-ivu',
        'application/vnd.ims.imsccv1p1',
        'application/vnd.ims.imsccv1p2',
        'application/vnd.ims.imsccv1p3',
        'application/vnd.ims.lis.v2.result+json',
        'application/vnd.ims.lti.v2.toolconsumerprofile+json',
        'application/vnd.ims.lti.v2.toolproxy.id+json',
        'application/vnd.ims.lti.v2.toolproxy+json',
        'application/vnd.ims.lti.v2.toolsettings.simple+json',
        'application/vnd.ims.lti.v2.toolsettings+json',
        'application/vnd.informedcontrol.rms+xml',
        'application/vnd.informix-visionary',
        'application/vnd.infotech.project',
        'application/vnd.infotech.project+xml',
        'application/vnd.innopath.wamp.notification',
        'application/vnd.insors.igm',
        'application/vnd.intercon.formnet',
        'application/vnd.intergeo',
        'application/vnd.intertrust.digibox',
        'application/vnd.intertrust.nncp',
        'application/vnd.intu.qbo',
        'application/vnd.intu.qfx',
        'application/vnd.iptc.g2.catalogitem+xml',
        'application/vnd.iptc.g2.conceptitem+xml',
        'application/vnd.iptc.g2.knowledgeitem+xml',
        'application/vnd.iptc.g2.newsitem+xml',
        'application/vnd.iptc.g2.newsmessage+xml',
        'application/vnd.iptc.g2.packageitem+xml',
        'application/vnd.iptc.g2.planningitem+xml',
        'application/vnd.ipunplugged.rcprofile',
        'application/vnd.irepository.package+xml',
        'application/vnd.is-xpr',
        'application/vnd.isac.fcs',
        'application/vnd.jam',
        'application/vnd.japannet-directory-service',
        'application/vnd.japannet-jpnstore-wakeup',
        'application/vnd.japannet-payment-wakeup',
        'application/vnd.japannet-registration',
        'application/vnd.japannet-registration-wakeup',
        'application/vnd.japannet-setstore-wakeup',
        'application/vnd.japannet-verification',
        'application/vnd.japannet-verification-wakeup',
        'application/vnd.jcp.javame.midlet-rms',
        'application/vnd.jisp',
        'application/vnd.joost.joda-archive',
        'application/vnd.jsk.isdn-ngn',
        'application/vnd.kahootz',
        'application/vnd.kde.karbon',
        'application/vnd.kde.kchart',
        'application/vnd.kde.kformula',
        'application/vnd.kde.kivio',
        'application/vnd.kde.kontour',
        'application/vnd.kde.kpresenter',
        'application/vnd.kde.kspread',
        'application/vnd.kde.kword',
        'application/vnd.kenameaapp',
        'application/vnd.kidspiration',
        'application/vnd.Kinar',
        'application/vnd.koan',
        'application/vnd.kodak-descriptor',
        'application/vnd.las.las+xml',
        'application/vnd.liberty-request+xml',
        'application/vnd.llamagraphics.life-balance.desktop',
        'application/vnd.llamagraphics.life-balance.exchange+xml',
        'application/vnd.lotus-1-2-3',
        'application/vnd.lotus-approach',
        'application/vnd.lotus-freelance',
        'application/vnd.lotus-notes',
        'application/vnd.lotus-organizer',
        'application/vnd.lotus-screencam',
        'application/vnd.lotus-wordpro',
        'application/vnd.macports.portpkg',
        'application/vnd.marlin.drm.actiontoken+xml',
        'application/vnd.marlin.drm.conftoken+xml',
        'application/vnd.marlin.drm.license+xml',
        'application/vnd.marlin.drm.mdcf',
        'application/vnd.mason+json',
        'application/vnd.maxmind.maxmind-db',
        'application/vnd.mcd',
        'application/vnd.medcalcdata',
        'application/vnd.mediastation.cdkey',
        'application/vnd.meridian-slingshot',
        'application/vnd.MFER',
        'application/vnd.mfmp',
        'application/vnd.micro+json',
        'application/vnd.micrografx.flo',
        'application/vnd.micrografx.igx',
        'application/vnd.microsoft.portable-executable',
        'application/vnd.miele+json',
        'application/vnd.mif',
        'application/vnd.minisoft-hp3000-save',
        'application/vnd.mitsubishi.misty-guard.trustweb',
        'application/vnd.Mobius.DAF',
        'application/vnd.Mobius.DIS',
        'application/vnd.Mobius.MBK',
        'application/vnd.Mobius.MQY',
        'application/vnd.Mobius.MSL',
        'application/vnd.Mobius.PLC',
        'application/vnd.Mobius.TXF',
        'application/vnd.mophun.application',
        'application/vnd.mophun.certificate',
        'application/vnd.motorola.flexsuite',
        'application/vnd.motorola.flexsuite.adsi',
        'application/vnd.motorola.flexsuite.fis',
        'application/vnd.motorola.flexsuite.gotap',
        'application/vnd.motorola.flexsuite.kmr',
        'application/vnd.motorola.flexsuite.ttc',
        'application/vnd.motorola.flexsuite.wem',
        'application/vnd.motorola.iprm',
        'application/vnd.mozilla.xul+xml',
        'application/vnd.ms-3mfdocument',
        'application/vnd.ms-artgalry',
        'application/vnd.ms-asf',
        'application/vnd.ms-cab-compressed',
        'application/vnd.ms-excel',
        'application/vnd.ms-excel.addin.macroEnabled.12',
        'application/vnd.ms-excel.sheet.binary.macroEnabled.12',
        'application/vnd.ms-excel.sheet.macroEnabled.12',
        'application/vnd.ms-excel.template.macroEnabled.12',
        'application/vnd.ms-fontobject',
        'application/vnd.ms-htmlhelp',
        'application/vnd.ms-ims',
        'application/vnd.ms-lrm',
        'application/vnd.ms-office.activeX+xml',
        'application/vnd.ms-officetheme',
        'application/vnd.ms-playready.initiator+xml',
        'application/vnd.ms-powerpoint',
        'application/vnd.ms-powerpoint.addin.macroEnabled.12',
        'application/vnd.ms-powerpoint.presentation.macroEnabled.12',
        'application/vnd.ms-powerpoint.slide.macroEnabled.12',
        'application/vnd.ms-powerpoint.slideshow.macroEnabled.12',
        'application/vnd.ms-powerpoint.template.macroEnabled.12',
        'application/vnd.ms-project',
        'application/vnd.ms-tnef',
        'application/vnd.ms-windows.printerpairing',
        'application/vnd.ms-wmdrm.lic-chlg-req',
        'application/vnd.ms-wmdrm.lic-resp',
        'application/vnd.ms-wmdrm.meter-chlg-req',
        'application/vnd.ms-wmdrm.meter-resp',
        'application/vnd.ms-word.document.macroEnabled.12',
        'application/vnd.ms-word.template.macroEnabled.12',
        'application/vnd.ms-works',
        'application/vnd.ms-wpl',
        'application/vnd.ms-xpsdocument',
        'application/vnd.msa-disk-image',
        'application/vnd.mseq',
        'application/vnd.msign',
        'application/vnd.multiad.creator',
        'application/vnd.multiad.creator.cif',
        'application/vnd.music-niff',
        'application/vnd.musician',
        'application/vnd.muvee.style',
        'application/vnd.mynfc',
        'application/vnd.ncd.control',
        'application/vnd.ncd.reference',
        'application/vnd.nervana',
        'application/vnd.netfpx',
        'application/vnd.neurolanguage.nlu',
        'application/vnd.nintendo.nitro.rom',
        'application/vnd.nintendo.snes.rom',
        'application/vnd.nitf',
        'application/vnd.noblenet-directory',
        'application/vnd.noblenet-sealer',
        'application/vnd.noblenet-web',
        'application/vnd.nokia.catalogs',
        'application/vnd.nokia.conml+wbxml',
        'application/vnd.nokia.conml+xml',
        'application/vnd.nokia.iptv.config+xml',
        'application/vnd.nokia.iSDS-radio-presets',
        'application/vnd.nokia.landmark+wbxml',
        'application/vnd.nokia.landmark+xml',
        'application/vnd.nokia.landmarkcollection+xml',
        'application/vnd.nokia.n-gage.ac+xml',
        'application/vnd.nokia.n-gage.data',
        'application/vnd.nokia.n-gage.symbian.install',
        'application/vnd.nokia.ncd',
        'application/vnd.nokia.pcd+wbxml',
        'application/vnd.nokia.pcd+xml',
        'application/vnd.nokia.radio-preset',
        'application/vnd.nokia.radio-presets',
        'application/vnd.novadigm.EDM',
        'application/vnd.novadigm.EDX',
        'application/vnd.novadigm.EXT',
        'application/vnd.ntt-local.content-share',
        'application/vnd.ntt-local.file-transfer',
        'application/vnd.ntt-local.ogw_remote-access',
        'application/vnd.ntt-local.sip-ta_remote',
        'application/vnd.ntt-local.sip-ta_tcp_stream',
        'application/vnd.oasis.opendocument.chart',
        'application/vnd.oasis.opendocument.chart-template',
        'application/vnd.oasis.opendocument.database',
        'application/vnd.oasis.opendocument.formula',
        'application/vnd.oasis.opendocument.formula-template',
        'application/vnd.oasis.opendocument.graphics',
        'application/vnd.oasis.opendocument.graphics-template',
        'application/vnd.oasis.opendocument.image',
        'application/vnd.oasis.opendocument.image-template',
        'application/vnd.oasis.opendocument.presentation',
        'application/vnd.oasis.opendocument.presentation-template',
        'application/vnd.oasis.opendocument.spreadsheet',
        'application/vnd.oasis.opendocument.spreadsheet-template',
        'application/vnd.oasis.opendocument.text',
        'application/vnd.oasis.opendocument.text-master',
        'application/vnd.oasis.opendocument.text-template',
        'application/vnd.oasis.opendocument.text-web',
        'application/vnd.obn',
        'application/vnd.oftn.l10n+json',
        'application/vnd.oipf.contentaccessdownload+xml',
        'application/vnd.oipf.contentaccessstreaming+xml',
        'application/vnd.oipf.cspg-hexbinary',
        'application/vnd.oipf.dae.svg+xml',
        'application/vnd.oipf.dae.xhtml+xml',
        'application/vnd.oipf.mippvcontrolmessage+xml',
        'application/vnd.oipf.pae.gem',
        'application/vnd.oipf.spdiscovery+xml',
        'application/vnd.oipf.spdlist+xml',
        'application/vnd.oipf.ueprofile+xml',
        'application/vnd.oipf.userprofile+xml',
        'application/vnd.olpc-sugar',
        'application/vnd.oma-scws-config',
        'application/vnd.oma-scws-http-request',
        'application/vnd.oma-scws-http-response',
        'application/vnd.oma.bcast.associated-procedure-parameter+xml',
        'application/vnd.oma.bcast.drm-trigger+xml',
        'application/vnd.oma.bcast.imd+xml',
        'application/vnd.oma.bcast.ltkm',
        'application/vnd.oma.bcast.notification+xml',
        'application/vnd.oma.bcast.provisioningtrigger',
        'application/vnd.oma.bcast.sgboot',
        'application/vnd.oma.bcast.sgdd+xml',
        'application/vnd.oma.bcast.sgdu',
        'application/vnd.oma.bcast.simple-symbol-container',
        'application/vnd.oma.bcast.smartcard-trigger+xml',
        'application/vnd.oma.bcast.sprov+xml',
        'application/vnd.oma.bcast.stkm',
        'application/vnd.oma.cab-address-book+xml',
        'application/vnd.oma.cab-feature-handler+xml',
        'application/vnd.oma.cab-pcc+xml',
        'application/vnd.oma.cab-subs-invite+xml',
        'application/vnd.oma.cab-user-prefs+xml',
        'application/vnd.oma.dcd',
        'application/vnd.oma.dcdc',
        'application/vnd.oma.dd2+xml',
        'application/vnd.oma.drm.risd+xml',
        'application/vnd.oma.group-usage-list+xml',
        'application/vnd.oma.pal+xml',
        'application/vnd.oma.poc.detailed-progress-report+xml',
        'application/vnd.oma.poc.final-report+xml',
        'application/vnd.oma.poc.groups+xml',
        'application/vnd.oma.poc.invocation-descriptor+xml',
        'application/vnd.oma.poc.optimized-progress-report+xml',
        'application/vnd.oma.push',
        'application/vnd.oma.scidm.messages+xml',
        'application/vnd.oma.xcap-directory+xml',
        'application/vnd.omads-email+xml',
        'application/vnd.omads-file+xml',
        'application/vnd.omads-folder+xml',
        'application/vnd.omaloc-supl-init',
        'application/vnd.openeye.oeb',
        'application/vnd.oracle.resource+json',
        'application/vnd.orange.indata',
        'application/vnd.osa.netdeploy',
        'application/vnd.osgeo.mapguide.package',
        'application/vnd.osgi.bundle',
        'application/vnd.osgi.dp',
        'application/vnd.osgi.subsystem',
        'application/vnd.otps.ct-kip+xml',
        'application/vnd.palm',
        'application/vnd.panoply',
        'application/vnd.paos.xml',
        'application/vnd.pawaafile',
        'application/vnd.pcos',
        'application/vnd.pg.format',
        'application/vnd.pg.osasli',
        'application/vnd.piaccess.application-licence',
        'application/vnd.picsel',
        'application/vnd.pmi.widget',
        'application/vnd.poc.group-advertisement+xml',
        'application/vnd.pocketlearn',
        'application/vnd.powerbuilder6',
        'application/vnd.powerbuilder6-s',
        'application/vnd.powerbuilder7',
        'application/vnd.powerbuilder7-s',
        'application/vnd.powerbuilder75',
        'application/vnd.powerbuilder75-s',
        'application/vnd.preminet',
        'application/vnd.previewsystems.box',
        'application/vnd.proteus.magazine',
        'application/vnd.publishare-delta-tree',
        'application/vnd.pvi.ptid1',
        'application/vnd.pwg-multiplexed',
        'application/vnd.pwg-xhtml-print+xml',
        'application/vnd.qualcomm.brew-app-res',
        'application/vnd.Quark.QuarkXPress',
        'application/vnd.quobject-quoxdocument',
        'application/vnd.radisys.moml+xml',
        'application/vnd.radisys.msml-audit-conf+xml',
        'application/vnd.radisys.msml-audit-conn+xml',
        'application/vnd.radisys.msml-audit-dialog+xml',
        'application/vnd.radisys.msml-audit-stream+xml',
        'application/vnd.radisys.msml-audit+xml',
        'application/vnd.radisys.msml-conf+xml',
        'application/vnd.radisys.msml-dialog-base+xml',
        'application/vnd.radisys.msml-dialog-fax-detect+xml',
        'application/vnd.radisys.msml-dialog-fax-sendrecv+xml',
        'application/vnd.radisys.msml-dialog-group+xml',
        'application/vnd.radisys.msml-dialog-speech+xml',
        'application/vnd.radisys.msml-dialog-transform+xml',
        'application/vnd.radisys.msml-dialog+xml',
        'application/vnd.radisys.msml+xml',
        'application/vnd.rainstor.data',
        'application/vnd.rapid',
        'application/vnd.realvnc.bed',
        'application/vnd.recordare.musicxml',
        'application/vnd.recordare.musicxml+xml',
        'application/vnd.RenLearn.rlprint',
        'application/vnd.rig.cryptonote',
        'application/vnd.route66.link66+xml',
        'application/vnd.rs-274x',
        'application/vnd.ruckus.download',
        'application/vnd.s3sms',
        'application/vnd.sailingtracker.track',
        'application/vnd.sbm.cid',
        'application/vnd.sbm.mid2',
        'application/vnd.scribus',
        'application/vnd.sealed.3df',
        'application/vnd.sealed.csf',
        'application/vnd.sealed.doc',
        'application/vnd.sealed.eml',
        'application/vnd.sealed.mht',
        'application/vnd.sealed.net',
        'application/vnd.sealed.ppt',
        'application/vnd.sealed.tiff',
        'application/vnd.sealed.xls',
        'application/vnd.sealedmedia.softseal.html',
        'application/vnd.sealedmedia.softseal.pdf',
        'application/vnd.seemail',
        'application/vnd.sema',
        'application/vnd.semd',
        'application/vnd.semf',
        'application/vnd.shana.informed.formdata',
        'application/vnd.shana.informed.formtemplate',
        'application/vnd.shana.informed.interchange',
        'application/vnd.shana.informed.package',
        'application/vnd.SimTech-MindMapper',
        'application/vnd.siren+json',
        'application/vnd.smaf',
        'application/vnd.smart.notebook',
        'application/vnd.smart.teacher',
        'application/vnd.software602.filler.form-xml-zip',
        'application/vnd.software602.filler.form+xml',
        'application/vnd.solent.sdkm+xml',
        'application/vnd.spotfire.dxp',
        'application/vnd.spotfire.sfs',
        'application/vnd.sss-cod',
        'application/vnd.sss-dtf',
        'application/vnd.sss-ntf',
        'application/vnd.stepmania.package',
        'application/vnd.stepmania.stepchart',
        'application/vnd.street-stream',
        'application/vnd.sun.wadl+xml',
        'application/vnd.sus-calendar',
        'application/vnd.svd',
        'application/vnd.swiftview-ics',
        'application/vnd.syncml.dm.notification',
        'application/vnd.syncml.dm+wbxml',
        'application/vnd.syncml.dm+xml',
        'application/vnd.syncml.dmddf+wbxml',
        'application/vnd.syncml.dmddf+xml',
        'application/vnd.syncml.dmtnds+wbxml',
        'application/vnd.syncml.dmtnds+xml',
        'application/vnd.syncml.ds.notification',
        'application/vnd.syncml+xml',
        'application/vnd.tao.intent-module-archive',
        'application/vnd.tcpdump.pcap',
        'application/vnd.tmd.mediaflex.api+xml',
        'application/vnd.tmobile-livetv',
        'application/vnd.trid.tpt',
        'application/vnd.triscape.mxs',
        'application/vnd.trueapp',
        'application/vnd.truedoc',
        'application/vnd.ubisoft.webplayer',
        'application/vnd.ufdl',
        'application/vnd.uiq.theme',
        'application/vnd.umajin',
        'application/vnd.unity',
        'application/vnd.uoml+xml',
        'application/vnd.uplanet.alert',
        'application/vnd.uplanet.alert-wbxml',
        'application/vnd.uplanet.bearer-choice',
        'application/vnd.uplanet.bearer-choice-wbxml',
        'application/vnd.uplanet.cacheop',
        'application/vnd.uplanet.cacheop-wbxml',
        'application/vnd.uplanet.channel',
        'application/vnd.uplanet.channel-wbxml',
        'application/vnd.uplanet.list',
        'application/vnd.uplanet.list-wbxml',
        'application/vnd.uplanet.listcmd',
        'application/vnd.uplanet.listcmd-wbxml',
        'application/vnd.uplanet.signal',
        'application/vnd.valve.source.material',
        'application/vnd.vcx',
        'application/vnd.vd-study',
        'application/vnd.vectorworks',
        'application/vnd.verimatrix.vcas',
        'application/vnd.vidsoft.vidconference',
        'application/vnd.visio',
        'application/vnd.visionary',
        'application/vnd.vividence.scriptfile',
        'application/vnd.vsf',
        'application/vnd.wap.sic',
        'application/vnd.wap.slc',
        'application/vnd.wap.wbxml',
        'application/vnd.wap.wmlc',
        'application/vnd.wap.wmlscriptc',
        'application/vnd.webturbo',
        'application/vnd.wfa.p2p',
        'application/vnd.wfa.wsc',
        'application/vnd.windows.devicepairing',
        'application/vnd.wmc',
        'application/vnd.wmf.bootstrap',
        'application/vnd.wolfram.mathematica',
        'application/vnd.wolfram.mathematica.package',
        'application/vnd.wolfram.player',
        'application/vnd.wordperfect',
        'application/vnd.wqd',
        'application/vnd.wrq-hp3000-labelled',
        'application/vnd.wt.stf',
        'application/vnd.wv.csp+wbxml',
        'application/vnd.wv.csp+xml',
        'application/vnd.wv.ssp+xml',
        'application/vnd.xacml+json',
        'application/vnd.xara',
        'application/vnd.xfdl',
        'application/vnd.xfdl.webform',
        'application/vnd.xmi+xml',
        'application/vnd.xmpie.cpkg',
        'application/vnd.xmpie.dpkg',
        'application/vnd.xmpie.plan',
        'application/vnd.xmpie.ppkg',
        'application/vnd.xmpie.xlim',
        'application/vnd.yamaha.hv-dic',
        'application/vnd.yamaha.hv-script',
        'application/vnd.yamaha.hv-voice',
        'application/vnd.yamaha.openscoreformat',
        'application/vnd.yamaha.openscoreformat.osfpvg+xml',
        'application/vnd.yamaha.remote-setup',
        'application/vnd.yamaha.smaf-audio',
        'application/vnd.yamaha.smaf-phrase',
        'application/vnd.yamaha.through-ngn',
        'application/vnd.yamaha.tunnel-udpencap',
        'application/vnd.yaoweme',
        'application/vnd.yellowriver-custom-menu',
        'application/vnd.zul',
        'application/vnd.zzazz.deck+xml',
        'application/voicexml+xml',
        'application/vq-rtcpxr',
        'application/watcherinfo+xml',
        'application/whoispp-query',
        'application/whoispp-response',
        'application/widget',
        'application/wita',
        'application/wordperfect5.1',
        'application/wsdl+xml',
        'application/wspolicy+xml',
        'application/x-www-form-urlencoded',
        'application/x400-bp',
        'application/xacml+xml',
        'application/xcap-att+xml',
        'application/xcap-caps+xml',
        'application/xcap-diff+xml',
        'application/xcap-el+xml',
        'application/xcap-error+xml',
        'application/xcap-ns+xml',
        'application/xcon-conference-info-diff+xml',
        'application/xcon-conference-info+xml',
        'application/xenc+xml',
        'application/xhtml-voice+xml',
        'application/xhtml+xml',
        'application/xml',
        'application/xml-dtd',
        'application/xml-external-parsed-entity',
        'application/xml-patch+xml',
        'application/xmpp+xml',
        'application/xop+xml',
        'application/xslt+xml',
        'application/xv+xml',
        'application/yang',
        'application/yin+xml',
        'application/zip',
        'application/zlib',
    ],
    'audio': [
        'audio/1d-interleaved-parityfec',
        'audio/32kadpcm',
        'audio/3gpp',
        'audio/3gpp2',
        'audio/ac3',
        'audio/AMR',
        'audio/AMR-WB',
        'audio/amr-wb+',
        'audio/aptx',
        'audio/asc',
        'audio/ATRAC-ADVANCED-LOSSLESS',
        'audio/ATRAC-X',
        'audio/ATRAC3',
        'audio/basic',
        'audio/BV16',
        'audio/BV32',
        'audio/clearmode',
        'audio/CN',
        'audio/DAT12',
        'audio/dls',
        'audio/dsr-es201108',
        'audio/dsr-es202050',
        'audio/dsr-es202211',
        'audio/dsr-es202212',
        'audio/DV',
        'audio/DVI4',
        'audio/eac3',
        'audio/encaprtp',
        'audio/EVRC',
        'audio/EVRC-QCP',
        'audio/EVRC0',
        'audio/EVRC1',
        'audio/EVRCB',
        'audio/EVRCB0',
        'audio/EVRCB1',
        'audio/EVRCNW',
        'audio/EVRCNW0',
        'audio/EVRCNW1',
        'audio/EVRCWB',
        'audio/EVRCWB0',
        'audio/EVRCWB1',
        'audio/example',
        'audio/fwdred',
        'audio/G719',
        'audio/G722',
        'audio/G7221',
        'audio/G723',
        'audio/G726-16',
        'audio/G726-24',
        'audio/G726-32',
        'audio/G726-40',
        'audio/G728',
        'audio/G729',
        'audio/G7291',
        'audio/G729D',
        'audio/G729E',
        'audio/GSM',
        'audio/GSM-EFR',
        'audio/GSM-HR-08',
        'audio/iLBC',
        'audio/ip-mr_v2.5',
        'audio/L16',
        'audio/L20',
        'audio/L24',
        'audio/L8',
        'audio/LPC',
        'audio/mobile-xmf',
        'audio/mp4',
        'audio/MP4A-LATM',
        'audio/MPA',
        'audio/mpa-robust',
        'audio/mpeg',
        'audio/mpeg4-generic',
        'audio/ogg',
        'audio/opus',
        'audio/parityfec',
        'audio/PCMA',
        'audio/PCMA-WB',
        'audio/PCMU',
        'audio/PCMU-WB',
        'audio/prs.sid',
        'audio/QCELP',
        'audio/raptorfec',
        'audio/RED',
        'audio/rtp-enc-aescm128',
        'audio/rtp-midi',
        'audio/rtploopback',
        'audio/rtx',
        'audio/SMV',
        'audio/SMV-QCP',
        'audio/SMV0',
        'audio/sp-midi',
        'audio/speex',
        'audio/t140c',
        'audio/t38',
        'audio/telephone-event',
        'audio/tone',
        'audio/UEMCLIP',
        'audio/ulpfec',
        'audio/VDVI',
        'audio/VMR-WB',
        'audio/vnd.3gpp.iufp',
        'audio/vnd.4SB',
        'audio/vnd.audiokoz',
        'audio/vnd.CELP',
        'audio/vnd.cisco.nse',
        'audio/vnd.cmles.radio-events',
        'audio/vnd.cns.anp1',
        'audio/vnd.cns.inf1',
        'audio/vnd.dece.audio',
        'audio/vnd.digital-winds',
        'audio/vnd.dlna.adts',
        'audio/vnd.dolby.heaac.1',
        'audio/vnd.dolby.heaac.2',
        'audio/vnd.dolby.mlp',
        'audio/vnd.dolby.mps',
        'audio/vnd.dolby.pl2',
        'audio/vnd.dolby.pl2x',
        'audio/vnd.dolby.pl2z',
        'audio/vnd.dolby.pulse.1',
        'audio/vnd.dra',
        'audio/vnd.dts',
        'audio/vnd.dts.hd',
        'audio/vnd.dvb.file',
        'audio/vnd.everad.plj',
        'audio/vnd.hns.audio',
        'audio/vnd.lucent.voice',
        'audio/vnd.ms-playready.media.pya',
        'audio/vnd.nokia.mobile-xmf',
        'audio/vnd.nortel.vbk',
        'audio/vnd.nuera.ecelp4800',
        'audio/vnd.nuera.ecelp7470',
        'audio/vnd.nuera.ecelp9600',
        'audio/vnd.octel.sbc',
        'audio/vnd.qcelp',
        'audio/vnd.rhetorex.32kadpcm',
        'audio/vnd.rip',
        'audio/vnd.sealedmedia.softseal.mpeg',
        'audio/vnd.vmx.cvsd',
        'audio/vorbis',
        'audio/vorbis-config',
    ],
    'image': [
        'image/cgm',
        'image/example',
        'image/fits',
        'image/g3fax',
        'image/gif',
        'image/ief',
        'image/jp2',
        'image/jpeg',
        'image/jpm',
        'image/jpx',
        'image/ktx',
        'image/naplps',
        'image/png',
        'image/prs.btif',
        'image/prs.pti',
        'image/pwg-raster',
        'image/svg+xml',
        'image/t38',
        'image/tiff',
        'image/tiff-fx',
        'image/vnd.adobe.photoshop',
        'image/vnd.airzip.accelerator.azv',
        'image/vnd.cns.inf2',
        'image/vnd.dece.graphic',
        'image/vnd.djvu',
        'image/vnd.dvb.subtitle',
        'image/vnd.dwg',
        'image/vnd.dxf',
        'image/vnd.fastbidsheet',
        'image/vnd.fpx',
        'image/vnd.fst',
        'image/vnd.fujixerox.edmics-mmr',
        'image/vnd.fujixerox.edmics-rlc',
        'image/vnd.globalgraphics.pgb',
        'image/vnd.microsoft.icon',
        'image/vnd.mix',
        'image/vnd.ms-modi',
        'image/vnd.net-fpx',
        'image/vnd.radiance',
        'image/vnd.sealed.png',
        'image/vnd.sealedmedia.softseal.gif',
        'image/vnd.sealedmedia.softseal.jpg',
        'image/vnd.svf',
        'image/vnd.tencent.tap',
        'image/vnd.valve.source.texture',
        'image/vnd.wap.wbmp',
        'image/vnd.xiff',
        'image/vnd.zbrush.pcx',
    ],
    'message': [
        'message/CPIM',
        'message/delivery-status',
        'message/disposition-notification',
        'message/example',
        'message/external-body',
        'message/feedback-report',
        'message/global',
        'message/global-delivery-status',
        'message/global-disposition-notification',
        'message/global-headers',
        'message/http',
        'message/imdn+xml',
        'message/news',
        'message/partial',
        'message/rfc822',
        'message/s-http',
        'message/sip',
        'message/sipfrag',
        'message/tracking-status',
        'message/vnd.si.simp',
        'message/vnd.wfa.wsc',
    ],
    'text': [
        'text/1d-interleaved-parityfec',
        'text/cache-manifest',
        'text/calendar',
        'text/css',
        'text/csv',
        'text/csv-schema',
        'text/dns',
        'text/ecmascript',
        'text/encaprtp',
        'text/enriched',
        'text/example',
        'text/fwdred',
        'text/grammar-ref-list',
        'text/html',
        'text/javascript',
        'text/jcr-cnd',
        'text/mizar',
        'text/n3',
        'text/parameters',
        'text/parityfec',
        'text/plain',
        'text/provenance-notation',
        'text/prs.fallenstein.rst',
        'text/prs.lines.tag',
        'text/raptorfec',
        'text/RED',
        'text/rfc822-headers',
        'text/richtext',
        'text/rtf',
        'text/rtp-enc-aescm128',
        'text/rtploopback',
        'text/rtx',
        'text/sgml',
        'text/t140',
        'text/tab-separated-values',
        'text/troff',
        'text/turtle',
        'text/ulpfec',
        'text/uri-list',
        'text/vcard',
        'text/vnd.a',
        'text/vnd.abc',
        'text/vnd.curl',
        'text/vnd.debian.copyright',
        'text/vnd.DMClientScript',
        'text/vnd.dvb.subtitle',
        'text/vnd.esmertec.theme-descriptor',
        'text/vnd.fly',
        'text/vnd.fmi.flexstor',
        'text/vnd.graphviz',
        'text/vnd.in3d.3dml',
        'text/vnd.in3d.spot',
        'text/vnd.IPTC.NewsML',
        'text/vnd.IPTC.NITF',
        'text/vnd.latex-z',
        'text/vnd.motorola.reflex',
        'text/vnd.ms-mediapackage',
        'text/vnd.net2phone.commcenter.command',
        'text/vnd.radisys.msml-basic-layout',
        'text/vnd.si.uricatalogue - OBSOLETED by request',
        'text/vnd.sun.j2me.app-descriptor',
        'text/vnd.trolltech.linguist',
        'text/vnd.wap.si',
        'text/vnd.wap.sl',
        'text/vnd.wap.wml',
        'text/vnd.wap.wmlscript',
        'text/xml',
        'text/xml-external-parsed-entity',
    ],
    'video': [
        'video/1d-interleaved-parityfec',
        'video/3gpp',
        'video/3gpp-tt',
        'video/3gpp2',
        'video/BMPEG',
        'video/BT656',
        'video/CelB',
        'video/DV',
        'video/encaprtp',
        'video/example',
        'video/H261',
        'video/H263',
        'video/H263-1998',
        'video/H263-2000',
        'video/H264',
        'video/H264-RCDO',
        'video/H264-SVC',
        'video/iso.segment',
        'video/JPEG',
        'video/jpeg2000',
        'video/mj2',
        'video/MP1S',
        'video/MP2P',
        'video/MP2T',
        'video/mp4',
        'video/MP4V-ES',
        'video/mpeg',
        'video/mpeg4-generic',
        'video/MPV',
        'video/nv',
        'video/ogg',
        'video/parityfec',
        'video/pointer',
        'video/quicktime',
        'video/raptorfec',
        'video/raw',
        'video/rtp-enc-aescm128',
        'video/rtploopback',
        'video/rtx',
        'video/SMPTE292M',
        'video/ulpfec',
        'video/vc1',
        'video/vnd.CCTV',
        'video/vnd.dece.hd',
        'video/vnd.dece.mobile',
        'video/vnd.dece.mp4',
        'video/vnd.dece.pd',
        'video/vnd.dece.sd',
        'video/vnd.dece.video',
        'video/vnd.directv.mpeg',
        'video/vnd.directv.mpeg-tts',
        'video/vnd.dlna.mpeg-tts',
        'video/vnd.dvb.file',
        'video/vnd.fvt',
        'video/vnd.hns.video',
        'video/vnd.iptvforum.1dparityfec-1010',
        'video/vnd.iptvforum.1dparityfec-2005',
        'video/vnd.iptvforum.2dparityfec-1010',
        'video/vnd.iptvforum.2dparityfec-2005',
        'video/vnd.iptvforum.ttsavc',
        'video/vnd.iptvforum.ttsmpeg2',
        'video/vnd.motorola.video',
        'video/vnd.motorola.videop',
        'video/vnd.mpegurl',
        'video/vnd.ms-playready.media.pyv',
        'video/vnd.nokia.interleaved-multimedia',
        'video/vnd.nokia.videovoip',
        'video/vnd.objectvideo',
        'video/vnd.radgamettools.bink',
        'video/vnd.radgamettools.smacker',
        'video/vnd.sealed.mpeg1',
        'video/vnd.sealed.mpeg4',
        'video/vnd.sealed.swf',
        'video/vnd.sealedmedia.softseal.mov',
        'video/vnd.uvvu.mp4',
        'video/vnd.vivo',
    ],
}



"""Provides all at one."""

import inspect
from typing import List

from mimesis.providers.address import Address
from mimesis.providers.base import BaseDataProvider
from mimesis.providers.business import Business
from mimesis.providers.clothing import ClothingSize
from mimesis.providers.code import Code
from mimesis.providers.cryptographic import Cryptographic
from mimesis.providers.date import Datetime
from mimesis.providers.development import Development
from mimesis.providers.file import File
from mimesis.providers.food import Food
from mimesis.providers.games import Games
from mimesis.providers.hardware import Hardware
from mimesis.providers.internet import Internet
from mimesis.providers.numbers import Numbers
from mimesis.providers.path import Path
from mimesis.providers.payment import Payment
from mimesis.providers.person import Person
from mimesis.providers.science import Science
from mimesis.providers.structure import Structure
from mimesis.providers.text import Text
from mimesis.providers.transport import Transport
from mimesis.providers.units import UnitSystem

__all__ = ['Generic']


class Generic(BaseDataProvider):
    """Class which contain all providers at one."""

    def __init__(self, *args, **kwargs):
        """Initialize attributes lazily.

        :param args: Arguments.
        :param kwargs: Keyword arguments.
        """
        super().__init__(*args, **kwargs)
        self._person = Person
        self._address = Address
        self._datetime = Datetime
        self._business = Business
        self._text = Text
        self._food = Food
        self._science = Science
        self._code = Code
        self._transport = Transport
        self.unit_system = UnitSystem(seed=self.seed)
        self.file = File(seed=self.seed)
        self.numbers = Numbers(seed=self.seed)
        self.development = Development(seed=self.seed)
        self.hardware = Hardware(seed=self.seed)
        self.clothing_size = ClothingSize(seed=self.seed)
        self.internet = Internet(seed=self.seed)
        self.path = Path(seed=self.seed)
        self.payment = Payment(seed=self.seed)
        self.games = Games(seed=self.seed)
        self.cryptographic = Cryptographic(seed=self.seed)
        self.structure = Structure(seed=self.seed)

    def __getattr__(self, attrname: str):
        """Get attribute without underscore.

        :param attrname: Attribute name.
        :return: An attribute.
        """
        attribute = object.__getattribute__(self, '_' + attrname)
        if attribute and callable(attribute):
            self.__dict__[attrname] = attribute(
                locale=self.locale,
                seed=self.seed,
            )
            return self.__dict__[attrname]

    def __dir__(self) -> List[str]:
        """Available data providers.

        :return: List of attributes.
        """
        attributes = []
        exclude = BaseDataProvider().__dict__.keys()

        for a in self.__dict__:
            if a not in exclude:
                if a.startswith('_'):
                    attribute = a.replace('_', '', 1)
                    attributes.append(attribute)
                else:
                    attributes.append(a)
        return attributes

    def add_provider(self, cls) -> None:
        """Add a custom provider to Generic() object.

        :param cls: Custom provider.
        :return: None
        :raises TypeError: if cls is not class.
        """
        if inspect.isclass(cls):
            name = ''
            if hasattr(cls, 'Meta'):
                if inspect.isclass(cls.Meta) and hasattr(cls.Meta, 'name'):
                    name = cls.Meta.name
            else:
                name = cls.__name__.lower()
            setattr(self, name, cls())
        else:
            raise TypeError('Provider must be a class')

    def add_providers(self, *providers) -> None:
        """Add a lot of custom providers to Generic() object.

        :param providers: Custom providers.
        :return: None
        """
        for provider in providers:
            self.add_provider(provider)
